from PymoNNto.Exploration.Network_UI.TabBase import *

class hist_tab(TabBase):

    def __init__(self, weight_attr='W', title='Weight Dist.', timesteps=1000, mask_param='Input_Mask', mask_color_add=(-100, -100, -100)):#mask_param=None #
        super().__init__(title)
        self.weight_attr = weight_attr
        self.timesteps = timesteps
        self.mask_param = mask_param
        if self.mask_param is not None:
            self.compiled_param = compile('n.'+self.mask_param, '<string>', 'eval')
            self.inverted_compiled_param = compile('np.invert(n.' + self.mask_param+')', '<string>', 'eval')
            self.mask_color_add = mask_color_add


    def add_recorder_variables(self, neuron_group, Network_UI):
        return
        #if hasattr(neuron_group, 'output'):
        #    Network_UI.add_recording_variable(neuron_group, 'n.output', self.timesteps)

    def initialize(self, Network_UI):
        self.additionaltab = Network_UI.Next_Tab(self.title)

        self.weight_hist_plots = {}
        self.net_weight_hist_plots = {}
        if self.mask_param is not None:
            self.net_inp_weight_hist_plots = {}

        for i,transmitter in enumerate(Network_UI.transmitters):
            if i>0:
                Network_UI.Next_H_Block()
            _, self.weight_hist_plots[transmitter] = Network_UI.Add_plot_curve(transmitter + ' weight hist', True, False, legend=False, x_label=transmitter + ' synapse size', y_label='Frequency')
            _, self.net_weight_hist_plots[transmitter] = Network_UI.Add_plot_curve(transmitter + ' network weight hist', True, False, legend=False, x_label=transmitter + ' synapse size', y_label='Frequency')
            if self.mask_param is not None:
                _, self.net_inp_weight_hist_plots[transmitter] = Network_UI.Add_plot_curve(transmitter + ' network input weight hist', True, False, legend=False, x_label=transmitter + ' synapse size', y_label='Frequency')

        #Network_UI.Next_H_Block()
        #_, self.gaba_weight_hist_plt = Network_UI.Add_plot_curve('GABA weight hist', True, False, legend=False)
        #_, self.gaba_net_weight_hist_plt = Network_UI.Add_plot_curve('GABA network weight hist', True, False, legend=False)
        #_, self.gaba_net_inp_weight_hist_plt = Network_UI.Add_plot_curve('GABA network input weight hist', True, False, legend=False)

        Network_UI.Next_H_Block()
        self.min_hist_slider = QSlider(1)  # QtCore.Horizontal
        self.min_hist_slider.setMinimum(-1)
        self.min_hist_slider.setMaximum(10)
        self.min_hist_slider.setSliderPosition(0)
        self.min_hist_slider.mouseReleaseEvent = Network_UI.static_update_func
        self.min_hist_slider.setToolTip('slide to cut away smallest weights')
        Network_UI.Add_element(self.min_hist_slider)  # , stretch=0.1

        Network_UI.Next_H_Block()
        self.bin_slider = QSlider(1)  # QtCore.Horizontal
        self.bin_slider.setMinimum(1)
        self.bin_slider.setMaximum(100)
        self.bin_slider.setSliderPosition(50)
        self.bin_slider.mouseReleaseEvent = Network_UI.static_update_func
        self.bin_slider.setToolTip('slide to change bin count')
        Network_UI.Add_element(self.bin_slider)  # , stretch=0.1

        # self.Next_H_Block()

        #def wnwi_click(event):
        #    image = get_whole_Network_weight_image(Network_UI.network[Network_UI.exc_group_name, 0], neuron_src_groups=None, individual_norm=True, exc_weight_attr='W', inh_weight_attr='W', activations=Network_UI.network[Network_UI.exc_group_name, 0].output)
        #    plt.imshow(image, interpolation="nearest")
        #    plt.show()

        #self.wnwi_btn = QPushButton('whole network weight image', Network_UI.main_window)
        #self.wnwi_btn.clicked.connect(wnwi_click)
        #Network_UI.Add_element(self.wnwi_btn)

        # self.Next_H_Block()

        # def ttp1_click(event):
        #    plot_t_vs_tp1(np.mean(np.array(self.selected_neuron_group()['n.output', 0][-1000:]), axis=1))
        # self.ttp1_btn = QPushButton('net t vs t+1 plot (1k)', self.main_window)
        # self.ttp1_btn.clicked.connect(ttp1_click)
        # self.Add_element(self.ttp1_btn)

        # def ives_click(event):
        #    inh=np.array(self.selected_neuron_group()['n.inhibition', 0][-1000:])
        #    exc=np.array(self.selected_neuron_group()['n.excitation', 0][-1000:])
        #    inhibition_excitation_scatter(inh,exc)
        # self.ives_btn = QPushButton('net inhibition vs excitation scatter (1k)', self.main_window)
        # self.ives_btn.clicked.connect(ives_click)
        # self.Add_element(self.ives_btn)



    def update_Synapse_Historgrams(self, Network_UI, group, input_mask, not_input_mask, net_color_input):
        msl = self.min_hist_slider.sliderPosition() * 0.001

        bins = self.bin_slider.sliderPosition()

        for transmitter in Network_UI.transmitters:
            if self.mask_param is not None:
                self.net_inp_weight_hist_plots[transmitter].clear()
            self.net_weight_hist_plots[transmitter].clear()
            self.weight_hist_plots[transmitter].clear()

            glu_syns = group.afferent_synapses[transmitter]
            if len(glu_syns) > 0:
                GLU_syn_list = get_combined_syn_mats(glu_syns, None, self.weight_attr)
                GLU_syn_list_en = get_combined_syn_mats(glu_syns, None, "enabled")
                if len(GLU_syn_list) > 0:
                    GLU_syn = GLU_syn_list[list(GLU_syn_list.keys())[0]]
                    en_mask = GLU_syn_list_en[list(GLU_syn_list_en.keys())[0]].astype(bool)*(GLU_syn > msl)

                    #GLU_syn = np.ma.array(GLU_syn, mask=en_mask)
                    #GLU_syn = GLU_syn.flatten()
                    #en_mask = en_mask.flatten()
                    #print(GLU_syn.shape, en_mask.shape)
                    #GLU_syn=GLU_syn[en_mask]


                    # print(GLU_syn.shape, selected_neuron_GLU_syn.shape)

                    # self.hist_plt.clear()
                    # y, x = np.histogram(np.sum(GLU_syn.transpose() > (np.max(GLU_syn, axis=1) * (1 / 2)), axis=0), bins=10)
                    # curve = pg.PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=(0, 0, 255, 255))
                    # self.hist_plt.addItem(curve)

                    if input_mask is not False and self.mask_param is not None:
                        self.net_inp_weight_hist_plots[transmitter].clear()
                        y, x = np.histogram(GLU_syn[input_mask][en_mask[input_mask]], bins=bins)#[GLU_syn[input_mask] > msl]
                        curve = pg.PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=net_color_input)
                        self.net_inp_weight_hist_plots[transmitter].addItem(curve)

                    self.net_weight_hist_plots[transmitter].clear()
                    y, x = np.histogram(GLU_syn[not_input_mask][en_mask[not_input_mask]], bins=bins)#[GLU_syn[not_input_mask] > msl]
                    curve = pg.PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=group.color)
                    self.net_weight_hist_plots[transmitter].addItem(curve)

                    self.weight_hist_plots[transmitter].clear()
                    y, x = np.histogram(GLU_syn[Network_UI.selected_neuron_id()][en_mask[Network_UI.selected_neuron_id()]], bins=bins)#[selected_neuron_GLU_syn > msl]
                    curve = pg.PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=Network_UI.neuron_select_color)
                    self.weight_hist_plots[transmitter].addItem(curve)


    def update(self, Network_UI):
        if self.additionaltab.isVisible():

            group = Network_UI.selected_neuron_group()
            n=group#for eval comand

            if self.mask_param is not None and hasattr(group, self.mask_param):
                input_mask = eval(self.compiled_param)
                not_input_mask = eval(self.inverted_compiled_param)
                mca = self.mask_color_add
            else:
                input_mask = False
                not_input_mask = True
                mca = (0,0,0)

            net_color_input = np.clip([group.color[0] + mca[0], group.color[1] + mca[1], group.color[2] + mca[2], 255], 0, 255)

            self.update_Synapse_Historgrams(Network_UI, group, input_mask, not_input_mask, net_color_input)




from PymoNNto.Exploration.Network_UI.Basic_Tabs.sidebar_fast_forward_module import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.sidebar_save_load_module import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.multi_group_plot_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.hist_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.info_tabs import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.single_group_plot_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.stability_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.weight_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.fourier_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.partition_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.spiketrain_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.individual_weight_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.PCA_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.isi_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.code_execution_tab import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.module_visualizer_tab import *

from PymoNNto.Exploration.Network_UI.Basic_Tabs.sidebar_neuron_grid import *
from PymoNNto.Exploration.Network_UI.Basic_Tabs.Analysis_Module_tab import *


def get_default_UI_modules(neuron_parameters=['output'], synapse_parameters=['W']):
    return [
    Analysis_Module_tab(),
    UI_sidebar_neuron_grid_module(1, neuron_parameters[0]),
    multi_group_plot_tab(neuron_parameters),#['output', 'TH', 'weight_norm_factor', 'nox', 'refractory_counter']
    spiketrain_tab(parameter=neuron_parameters[0]),
    weight_tab(weight_attrs=synapse_parameters),#, 'W_temp', 'W_stable'
    partition_tab(),
    PCA_tab(parameter=neuron_parameters[0]),
    #individual_weight_tab(),
    isi_tab(param=neuron_parameters[0]),
    hist_tab(weight_attr=synapse_parameters[0]),
    single_group_plot_tab(neuron_parameters),#, 'excitation':(0, 0, 255), 'inhibition':(255, 0, 0), 'input_act':(255, 0, 255), 'TH':(0, 255, 0)}),
    stability_tab(parameter=neuron_parameters[0]),
    fourier_tab(parameter=neuron_parameters[0]),
    info_tab(),
    sidebar_fast_forward_module(),
    sidebar_save_load_module(),
    code_execution_tab(),
    #module_visualizer_tab()
]

def get_modules_dict(*args): #get_modules_dict([m1,m2,m3],m4,[m5],{1:m6,2:m7},...)
    result = {}



    for arg in args:
        if type(arg) is list:
            for a in arg:
                result[a.__class__] = a

        elif type(arg) is dict:
            for k in arg:
                result[arg[k].__class__] = arg[k]

        else:
            result[arg.__class__] = arg


    #for module in modules:
    #    result[module.__class__.__name__] = module
    return result


from PymoNNto.NetworkBehaviour.Structure.Structure import *
from PymoNNto.NetworkCore.Behaviour import *
from PymoNNto.NetworkCore.Synapse_Group import *


class Remove_Autapses(Behaviour):
    set_variables_on_init = True

    def set_variables(self, synapses):
        src = synapses.src
        dst = synapses.dst

        s_id = np.tile(np.arange(src.size), (1, dst.size)).reshape(dst.size, src.size)
        d_id = np.tile(np.arange(dst.size), (1, src.size)).reshape(src.size, dst.size).transpose()

        synapses.enabled *= (s_id != d_id)


class Receptive_Fields(Behaviour):
    visualization_module_inputs = ['s.src.x', 's.src.y', 's.src.z', 's.dst.x', 's.dst.y', 's.dst.z']

    set_variables_on_init = True

    def get_enabled_mat(self, synapses, s_id, sx, sy, sz, d_id, dx, dy, dz):
        print('not implemented. Please use inherited classes below.')
        return synapses.enabled

    def set_variables(self, synapses):
        src = synapses.src
        dst = synapses.dst
        #synapses.enabled = synapses.get_synapse_mat()

        s_id = np.tile(np.arange(src.size), (1, dst.size)).reshape(dst.size, src.size)#vec_to_mat(np.arange(src.size), dst.size) #

        sx = None
        sy = None
        sz = None

        if hasattr(src, 'x') and hasattr(src, 'y') and hasattr(src, 'z'):
            #sx = vec_to_mat(src.x, dst.size)
            #sy = vec_to_mat(src.y, dst.size)
            #sz = vec_to_mat(src.z, dst.size)

            sx = np.tile(src.x, (1, dst.size)).reshape(dst.size, src.size)
            sy = np.tile(src.y, (1, dst.size)).reshape(dst.size, src.size)
            sz = np.tile(src.z, (1, dst.size)).reshape(dst.size, src.size)

        d_id = np.tile(np.arange(dst.size), (1, src.size)).reshape(src.size, dst.size).transpose()

        dx = None
        dy = None
        dz = None

        if hasattr(dst, 'x') and hasattr(dst, 'y') and hasattr(dst, 'z'):
            dx = np.tile(dst.x, (1, src.size)).reshape(src.size, dst.size).transpose()
            dy = np.tile(dst.y, (1, src.size)).reshape(src.size, dst.size).transpose()
            dz = np.tile(dst.z, (1, src.size)).reshape(src.size, dst.size).transpose()

        synapses.enabled *= self.get_enabled_mat(synapses, s_id, sx, sy, sz, d_id, dx, dy, dz)

        if self.get_init_attr('remove_autapses', False):
            synapses.enabled *= (s_id != d_id)


class Box_Receptive_Fields(Receptive_Fields):

    def get_enabled_mat(self, synapses, s_id, sx, sy, sz, d_id, dx, dy, dz):
        range=self.get_init_attr('range', None, required=True)
        if sx is not None and sy is not None and sz is not None and dx is not None and dy is not None and dz is not None:
            x_diff = np.abs(sx - dx)
            y_diff = np.abs(sy - dy)
            z_diff = np.abs(sz - dz)
            return (x_diff <= range) * (y_diff <= range) * (z_diff <= range)
        else:
            print('Neuron needs x,y and z vectors to create receptive fields')
            return synapses.enabled

class Circle_Receptive_Fields(Receptive_Fields):

    def get_enabled_mat(self, synapses, s_id, sx, sy, sz, d_id, dx, dy, dz):
        radius = self.get_init_attr('radius', None, required=True)
        if sx is not None and sy is not None and sz is not None and dx is not None and dy is not None and dz is not None:
            x_diff = np.abs(sx - dx)
            y_diff = np.abs(sy - dy)
            z_diff = np.abs(sz - dz)
            return np.sqrt(x_diff * x_diff + y_diff * y_diff + z_diff * z_diff) <= radius
        else:
            print('Neuron needs x,y and z vectors to create receptive fields')
            return synapses.enabled

#r = Circle_Receptive_Fields()
#r.visualize_module()



content = [
    "docs/index.md",
    "docs/Introduction/installation.md",
    "docs/Introduction/installation2.md",
    "docs/Tools/basics.md",
    "docs/Tools/Synapses_and_Input.md",
    "docs/Tools/User_Interface.md"]


f = open("readme.md", "w")

f.write("https://pymonnto.readthedocs.io/\r\n\r\n")

for c in content:
    text = open(c, "r").read()
    print(text)
    f.write(text)
    f.write("\r\n")


f.close()




from PymoNNto.NetworkCore.Behaviour import *
import numpy as np
import tensorflow as tf

class Synaptic_Normalization_TF(Behaviour):

    def set_variables(self, neurons):
        self.syn_type = self.get_init_attr('syn_type', 'GLU', neurons)

        neurons.require_synapses(self.syn_type, warning=False)#suppresses error when synapse group does not exist

        self.clip_min = self.get_init_attr('clip_min', 0.0, neurons)
        self.clip_max = self.get_init_attr('clip_max', 1000000.0, neurons)

        self.norm_factor = tf.constant(self.get_init_attr('norm_factor', 1.0, neurons), dtype='float32')

        neurons.temp_weight_sum = tf.Variable(neurons.get_neuron_vec(), dtype='float32')

    def new_iteration(self, neurons):
        neurons.temp_weight_sum.assign(tf.multiply(neurons.temp_weight_sum, 0.0))

        for s in neurons.afferent_synapses[self.syn_type]:
            s.dst.temp_weight_sum.assign(tf.add(s.dst.temp_weight_sum, tf.math.reduce_sum(s.W, axis=1)))

        neurons.temp_weight_sum.assign(tf.divide(neurons.temp_weight_sum, self.norm_factor))

        for s in neurons.afferent_synapses[self.syn_type]:
            #d = tf.add(s.dst.temp_weight_sum, tf.cast(tf.math.equal(s.dst.temp_weight_sum, 0.0), dtype='float32'))
            d = s.dst.temp_weight_sum
            s.W.assign(tf.transpose(tf.divide(tf.transpose(s.W), d)))



from PymoNNto.Exploration.Network_UI.TabBase import *
from PymoNNto.Exploration.StorageManager.StorageManager import *

def pol2cart(theta, rho):
    x = rho * np.cos(theta)
    y = rho * np.sin(theta)
    return x, y

def cart2pol(x, y):
    theta = np.arctan2(y, x)
    rho = np.hypot(x, y)
    return theta, rho

class DrawItem2(pg.GraphicsObject):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.picture = QtGui.QPicture()

    #base_char_masks_dict

    def get_axis_position(self, axis_id, number_of_axis, min_val, max_val, value):
        number_of_axis=max(number_of_axis,3)
        value_range=(max_val-min_val)
        if value_range==0:
            value_range=1.0
        return pol2cart(axis_id / number_of_axis * 2 * np.pi, 100.0/value_range*(value-min_val))


    def draw_rings(self, painter, alphabet, num_rings):


        for i in range(num_rings):
            c=int(220)
            painter.setPen(pg.mkPen(color=(c, c, c, 255)))
            r = 100/num_rings*(i+1)#np.power(0.9, i)


            painter.setBrush(pg.mkBrush(color=(c, c, c, 0)))
            painter.drawEllipse(QtCore.QPointF(0, 0), r, r)
            '''
            for d, char in enumerate(alphabet):
                x, y = pol2cart(d / len(alphabet) * 2 * np.pi, r)
                painter.setBrush(pg.mkBrush(color=(c, c, c, 255)))
                painter.drawEllipse(QtCore.QPointF(x, y), 1, 1)

                if i == 0:
                    painter.drawLine(QtCore.QPointF(0, 0), QtCore.QPointF(x, y))
            '''

        for d, char in enumerate(alphabet):
            x, y = self.get_axis_position(d, len(alphabet),0,100,100)
            #pol2cart(d / len(alphabet) * 2 * np.pi, 100)
            painter.setBrush(pg.mkBrush(color=(0, 0, 0, 255)))
            painter.setPen(pg.mkPen(color=(0, 0, 0, 255)))
            painter.drawLine(QtCore.QPointF(0, 0), QtCore.QPointF(x, y))


    #def fixate_points(self, group, alphabet, input_mat):
    #    for i, char in enumerate(alphabet):
    #        bm=input_mat[:, i]
    #        x, y = pol2cart(i / len(alphabet) * 2 * np.pi, 100)
    #        group.buffer_posx[bm > 0.1] = x
    #        group.buffer_posy[bm > 0.1] = y

    def draw_chars(self, painter, alphabet, data):
        for i, char in enumerate(alphabet):
            x, y = self.get_axis_position(i, len(alphabet),0,100,110)#pol2cart(i / len(alphabet) * 2 * np.pi, 110)

            #min_a = data.min(axis=1)
            #max_a = data.max(axis=1)

            qf = QFont('Arial')
            qf.setPointSizeF(5.0)
            #qf.setStretch(2)
            painter.setFont(qf)
            painter.drawText(x,y,char)#+' ('+str(min_a[i])+'-'+str(max_a[i])+')'

    def draw_genome(self, painter, alphabet, len_alphabet, min_a, max_a, vals):
        positions = []
        for i, a in enumerate(alphabet):
            positions.append(self.get_axis_position(i, len_alphabet, min_a[i], max_a[i], vals[i]))

        for p_i in range(len(positions)):
            if p_i != 0:
                painter.drawLine(QtCore.QPointF(positions[p_i - 1][0], positions[p_i - 1][1]),
                                 QtCore.QPointF(positions[p_i][0], positions[p_i][1]))
            else:
                painter.drawLine(QtCore.QPointF(positions[p_i][0], positions[p_i][1]),
                                 QtCore.QPointF(positions[-1][0], positions[-1][1]))

    def draw_genomes(self, alphabet, data, painter, max_gene_draw):

        #mask = data[-2, :] % np.ceil(data.shape[1]/100) == 0
        #masked_data=data[:, mask]
        # mask = [i%10 == 0 for i in range(data.shape[1])]

        if data.shape[1] > max_gene_draw:
            masked_data = data[:, -max_gene_draw:]
        else:
            masked_data = data

        min_a = data.min(axis=1)
        max_a = data.max(axis=1)

        masked_min_a = masked_data.min(axis=1)
        masked_max_a = masked_data.max(axis=1)

        la = len(alphabet)
        score_diff = (masked_max_a[-1] - masked_min_a[-1])

        if la > 0:
            for j in range(masked_data.shape[1]):
                if score_diff > 0:
                    c = int(255.0/score_diff*(masked_data[-1, j]-masked_min_a[-1]))
                else:
                    c = 0
                painter.setPen(pg.mkPen(color=(255-c, c, 0, 255), width=1))

                self.draw_genome(painter, alphabet, la, min_a, max_a, masked_data[:, j])

    def draw_clicked_genome(self, alphabet, data, painter, clicked_id):
        min_a = data.min(axis=1)
        max_a = data.max(axis=1)

        painter.setPen(pg.mkPen(color=(0, 0, 255, 255), width=3))
        self.draw_genome(painter, alphabet, len(alphabet), min_a, max_a, data[:, clicked_id])

    def update_pic(self, alphabet, data, clicked_id, max_gene_draw=100): #, groups, alphabet, p0, p1, p2, p3, p4, p5, nui, statistics, show_weights):

        self.data=data
        self.alphabet=alphabet

        self.picture = QtGui.QPicture()
        painter = QtGui.QPainter(self.picture)
        painter.scale(1, -1)

        self.draw_rings(painter, alphabet, 10)

        if data is not None:
            self.draw_genomes(alphabet, data, painter, max_gene_draw)

            if clicked_id!=-1:
                self.draw_clicked_genome(alphabet, data, painter, clicked_id)

            painter.setPen(pg.mkPen(color=(0, 0, 0, 255)))
            self.draw_chars(painter, alphabet, data)



        '''
        self.groups=groups

        #for group in groups:
        #    self.initialize_neuron_positons(group)

        for group in nui.network.NeuronGroups:
            self.compute_and_apply_attraction(group, p1, p2, p3, p4, p5, attractor_rads)

        painter.scale(1, -1)

        for group in groups:
            self.draw_neurons(painter, group, p0)

        #painter.setBrush(pg.mkBrush(color=(0,255,0,255)))
        #for x, y in zip(group.buffer_posx+attractor_rad_x, group.buffer_posy+attractor_rad_y):
        #    painter.drawEllipse(QtCore.QPointF(x, y), 1, 1)

        if show_weights:
            for group in groups:
                if group == nui.selected_neuron_group():
                    self.draw_weights(painter, group, nui.selected_neuron_id())


        painter.setPen(pg.mkPen(color=(0, 0, 0, 255), width=10))
        painter.setBrush(pg.mkBrush(color=(0, 0, 0, 0)))

        for group in nui.network.NeuronGroups:
            if hasattr(group, 'Input_Weights'):
                self.fixate_points(group, alphabet, group.Input_Weights)

        self.draw_chars(painter, alphabet, statistics)
        '''

        painter.end()
        self.prepareGeometryChange()
        self.informViewBoundsChanged()
        #self.update()

    def paint(self, painter, option, widget=None):
        painter.drawPicture(0, 0, self.picture)

    def boundingRect(self):
        br = self.picture.boundingRect()
        br.setTop(max(br.top()-10, br.left()-10))
        br.setLeft(min(br.top()-10, br.left()-10))
        br.setRight(max(br.right()+10, br.bottom()+10))
        br.setBottom(max(br.right()+10, br.bottom()+10))
        return QtCore.QRectF(br)

def add_evolution_plot_items(ui, tab):
    tab.curves, tab.plot = ui.Add_plot_curve(x_label='generations', y_label='score', number_of_curves=3, return_plot=True, colors=[(0, 0, 0), (0, 0, 0), (0, 0, 0)])
    tab.item = pg.FillBetweenItem(curve1=tab.curves[1], curve2=tab.curves[2], brush=(255, 0, 0, 100))
    tab.plot.addItem(tab.item)
    tab.scatter = pg.ScatterPlotItem(size=10, brush=pg.mkBrush(255, 255, 255, 120))
    tab.plot.addItem(tab.scatter)

    tab.clicked_generation=-1
    tab.clicked_score=-1
    tab.clicked_id=-1

    def clicked(plot, points):
        print('clicked')
        #for p in points:
        if len(points) > 0:
            p = points[-1]
            tab.clicked_generation = p._data[0]
            tab.clicked_score = p._data[1]
            tab.scatter2.setData(x=[tab.clicked_generation],y=[tab.clicked_score])

            if tab.data is not None and tab.gene_keys is not None:
                tab.clicked_id = np.where((tab.data[-2] == tab.clicked_generation) & (tab.data[-1] == tab.clicked_score))
                if len(tab.clicked_id) > 0:
                    tab.clicked_id = tab.clicked_id[0]
                    if len(tab.clicked_id) > 0:
                        tab.clicked_id = tab.clicked_id[0]
                        tab.clicked_evo.setText('Clicked genome: ' + str(dict(zip(tab.gene_keys + ['gen', 'score'], tab.data[:, tab.clicked_id]))))

                        tab.draw_item.update_pic(tab.gene_keys, tab.data, tab.clicked_id, tab.slider.value())

    tab.scatter.sigClicked.connect(clicked)

    tab.scatter2 = pg.ScatterPlotItem(size=10, brush=pg.mkBrush(0, 0, 255, 255))
    tab.plot.addItem(tab.scatter2)

    #g = pg.GraphItem()
    #tab.plot.addItem(g)
    # Update the graph
    #g.setData(pos=np.array([[100, 100]]), pen=['r'], brush=['r'], size=10, symbol=['o'], pxMode=False)

    tab.radar_plot = ui.Add_plot(x_label='radar plot', tooltip_message='radar plot of the last 100 genomes. (green=high score/red=low score/blue=selected)')  # axisItems={'left': stringaxis}
    tab.draw_item = DrawItem2()
    tab.radar_plot.addItem(tab.draw_item)

    def val_changed():
        if hasattr(tab, 'data'):
            tab.draw_item.update_pic(tab.gene_keys, tab.data, tab.clicked_id, tab.slider.value())

    tab.slider = ui.Add_element(QSlider(Qt.Vertical))
    tab.slider.setMinimum(10)
    tab.slider.setMaximum(5000)
    tab.slider.setValue(100)
    tab.slider.valueChanged.connect(val_changed)

    tab.radar_plot.hideAxis('bottom')
    tab.radar_plot.hideAxis('left')
    tab.draw_item.update_pic([], None, None, tab.slider.value())
    #tab.draw_item.mouseClickEvent = c



    ui.Next_H_Block(stretch=0.0)
    tab.best_evo = ui.Add_element(QLineEdit('...'), stretch=0)

    ui.Next_H_Block(stretch=0.0)
    tab.clicked_evo = ui.Add_element(QLineEdit('...'), stretch=0)



def update_evolution_plot(ui, tab, evo_name, gene_keys, data_folder=get_data_folder()):
    print(len(next(os.walk(data_folder + '/StorageManager/'+evo_name))[1]), 'datapoints found')

    smg = StorageManagerGroup(evo_name, data_folder=data_folder)

    smg.sort_by('gen')

    tab.gene_keys = gene_keys

    result_lists = smg.get_multi_param_list(gene_keys+['gen', 'score'], remove_None=True)

    xa, ya = smg.remove_duplicates_get_eval(result_lists[-2], result_lists[-1], evalstr='np.average(a)')
    xs, ymi = smg.remove_duplicates_get_eval(result_lists[-2], result_lists[-1], evalstr='np.min(a)')
    xs, yma = smg.remove_duplicates_get_eval(result_lists[-2], result_lists[-1], evalstr='np.max(a)')

    tab.curves[0].setData(xa, ya)
    tab.curves[1].setData(xa, ymi)
    tab.curves[2].setData(xa, yma)

    tab.scatter.setData(x=result_lists[-2], y=result_lists[-1])

    tab.data = np.array(result_lists)

    tab.best_evo.setText('Best genome: ' + str(dict(zip(gene_keys+['gen', 'score'], tab.data[:, -1]))))

    tab.draw_item.update_pic(gene_keys, tab.data, tab.clicked_id, tab.slider.value())

    tab.radar_plot.update()






from pathlib import Path

def py_to_md(py_file_name, md_file_name):

    if not Path(md_file_name).is_file():
        print('create', md_file_name)
        md_file = open(md_file_name, "x")
        md_text = '``` ```'
    else:
        md_file = open(md_file_name, "r")
        md_text = md_file.read()

    py_file = open(py_file_name, "r")
    py_text = py_file.read()

    #partial_py = py_text.split('\'\'\'\'\'')
    #description = ''
    #if len(partial_py) > 2:
    #    description = partial_py[1]
    #    print(description)

    text = '```python\r\n'+py_text+'\r\n```'#description+
    partial_md = md_text.split('```')
    if len(partial_md) > 2:
        partial_md[1] = text
    else:
        partial_md.append(text)
        print('not able to find code block in', md_file_name, '(add py file to end)')


    md_file.close()
    py_file.close()

    md_file = open(md_file_name, "w")

    for str in partial_md:
        md_file.write(str)

    md_file.close()



yml_file = open('mkdocs.yml', "r")

yml_text = yml_file.read()

for str in yml_text.split('\n'):
    if '.md' in str and '- ' in str and ':' in str and '.py' in str and '-->' in str and '<!---' in str:
        right_block=str.split('<!---')[1]
        while ' ' in right_block:
            right_block = right_block.replace(' ', '')
        right_block = right_block.replace('-->', '')

        files = right_block.split(':')

        print(files[0], '->', files[1])

        if len(files) == 2 and files[1][-3:] == '.md' and files[0][-3:] == '.py':
            py_to_md(files[0], 'docs/'+files[1])

yml_file.close()


'''
content = [
    "Brunel_Hakim",
    "Wang_Buzaki",
    "Diesmann_Synfire",
    "Hindmarsh_Rose",
    "Hodgkin_Huxley",
    "Hopfield",
    "Izhikevich",
    "LeakyIntegrateAndFire"]

for n in content:
    py_to_md(py_file_name='Examples_Classical/'+n+'.py', md_file_name='docs/Classical/'+n+'.md')





content = [
]

for n in content:
    py_to_md(py_file_name='Examples_Behaviour/'+n+'.py', md_file_name='docs/Behaviours/'+n+'.md')






content = [
]

for n in content:
    py_to_md(py_file_name='Examples_Paper/'+n+'.py', md_file_name='docs/Experimental/'+n+'.md')

#tutorial manual...




content = [
]

for n in content:
    py_to_md(py_file_name='Examples_UI/'+n+'.py', md_file_name='docs/UI/'+n+'.md')
    
    
'''






import sys
sys.path.append('../../../TREN2')
sys.path.append('../../../tren2')
from PymoNNto.Exploration.StorageManager.StorageManager import *


smg = StorageManagerGroup('SORN_Gram_train_time_100k')# SORN_One_layer_all_600

x=[]
y=[]

for ms_i, sm in enumerate(smg.StorageManagerList):
    try:
        network = sm.load_obj('net')
        source = sm.load_obj('source')
        network.NeuronGroups = [network.NeuronGroups[0]]
        network.SynapseGroups = [network.SynapseGroups[0]]
        groups = [network.NeuronGroups[0]]
        RALN = Reconstruct_Analyze_Label_Network(network)
        RALN.label_and_group_neurons(network.NeuronGroups[0],
                                     [source.get_activation(char) for char in range(source.get_alphabet_length())], 'W', 10)

        trans = RALN.get_synapse_transition_dict(source, x_attribute_name='temporal_layer',
                                                 y_attribute_name='class_label',
                                                 weight_attribute_name='W', groups=groups, weight_limit=1 / 3)

        freq, ocd, filtered_trans = RALN.get_transition_frequencies(trans, source)
        print(ocd)
        classes = RALN.get_transition_classes(trans, freq)

        x_n, y_n = RALN.plot_transition_frequencies(filtered_trans, freq, plot_single=True, plot_multiple=True, show_labels=True)

        print(x_n)

        x+=list(x_n)
        y+=list(y_n)


        #plt.show()
        # RALN.plot_bars(filtered_trans)
        # plt.xlabel('label')
        # plt.ylabel('synapses')
        # plt.show()
        # RALN.plot_bars(classes)
        # plt.xlabel('label')
        # plt.ylabel('synapses')
        # plt.show()

        #print(sm.absolute_path)
        #sm.save_param_dict()
    except Exception as e:
        print('not able to load network or source',e)
    if ms_i > 0:
        break

#print(np.array(x).flatten().shape,np.array(y).flatten().shape)

RALN.draw_trend_line(np.array(x).flatten(), np.array(y).flatten())

plt.xlabel('synapses')
plt.ylabel('found in text')
#plt.xscale('log')
#plt.xscale('log')
plt.show()
#todo fix text occurences vs num of synapses!!!
#todo


from PymoNNto.Exploration.Network_UI.TabBase import *


class info_tab(TabBase):

    def add_recorder_variables(self, neuron_group, Network_UI):
        return

    def initialize(self, Network_UI):

        Network_UI.infotabs=[]

        for group in Network_UI.network.NeuronGroups:
            self.make_group_tab(Network_UI, [group])

        temp = {}
        for g in Network_UI.network.SynapseGroups:
            key = ','.join(g.tags)
            if key not in temp:
                temp[key] = [g]
            else:
                temp[key].append(g)

        for k in temp:
            self.make_group_tab(Network_UI, temp[k])



    def make_group_tab(self, Network_UI, groups):
        title = '[' + str(groups[0].tags[0]) + ']'
        if len(groups)>1:
            title+=' (Partitioned)'
        infotab = Network_UI.Next_Tab(title)

        Network_UI.infotabs.append(infotab)

        h = 0

        caption = QLabel(str(groups[0].tags[0]) + ':')
        caption.setFont(QFont('SansSerif', 16))
        h += 40
        Network_UI.Add_element(caption)
        Network_UI.Next_H_Block()

        for key in groups[0].behaviour:
            main_tag = groups[0].behaviour[key].tags[0]

            behaviours = [g.behaviour[key] for g in groups]

            self.add_line(Network_UI, str(key) + ' ' + main_tag, behaviours, groups)
            h += 60

        infotab.setMaximumHeight(h)

    def add_line(self, Network_UI, text, behaviours=None, groups=None):

        element = QCheckBox()
        element.setChecked(behaviours[0].behaviour_enabled)
        element.setText(text)

        def click(event):
            for behaviour in behaviours:
                behaviour.behaviour_enabled = element.isChecked()

        def cut_length(s):
            result = ''
            for i, c in enumerate(s):
                result += c
                if i % 80 == 79:
                    result += '\r\n'
            return result

        element.stateChanged.connect(click)
        element2 = QLabel(cut_length(str(behaviours[0].init_kwargs)))

        container = QVBoxLayout()  # self.main_window

        def addArg(arg):
            h_container = QHBoxLayout()
            h_container.addWidget(QLabel(arg), stretch=1)

            edit = QLineEdit(str(behaviours[0].init_kwargs[arg]))
            edit.arg = arg

            def change_param(event):
                for behaviour in behaviours:
                    behaviour.init_kwargs[edit.arg] = edit.text()

            edit.textChanged.connect(change_param)

            h_container.addWidget(edit, stretch=1)
            h_container.addWidget(QLabel(''), stretch=5)
            container.addLayout(h_container)

        for arg in behaviours[0].init_kwargs:
            addArg(arg)

        def update(event):
            for neurons,behaviour in zip(groups, behaviours):
                behaviour.set_variables(neurons)
                element2.setText(cut_length(str(behaviour.init_kwargs)))

        h_container = QHBoxLayout()
        btn = QPushButton('update')  # , self.main_window
        btn.clicked.connect(update)
        h_container.addWidget(btn, stretch=2)
        h_container.addWidget(QLabel(''), stretch=5)
        container.addLayout(h_container)

        frame = QFrame()
        frame.setLayout(container)
        frame.hide()

        def hide_show(event):
            if frame.isVisible():
                frame.hide()
            else:
                frame.show()

        if not 'structure' in behaviours[0].tags:
            element2.mousePressEvent = hide_show
            link_font = QFont()  # 'SansSerif', 12
            link_font.setUnderline(True)
            # palette = QPalette()
            # palette.setColor(QPalette.Text, QtCore.Qt.blue)
            # element2.setPalette(palette)
            element2.setFont(link_font)
            element2.setStyleSheet("color: rgb(0,0,200)")

        Network_UI.Add_element(element)
        Network_UI.Add_element(element2, stretch=5)
        Network_UI.Next_H_Block()
        # self.current_H_block.addLayout(container)

        Network_UI.Add_element(frame)
        Network_UI.Next_H_Block()

    def update(self, Network_UI):
        return


from PymoNNto.NetworkCore.Behaviour import *
import numpy as np

class Synaptic_Normalization(Behaviour):

    def set_variables(self, neurons):
        self.syn_type = self.get_init_attr('syn_type', 'GLU', neurons)
        self.clip_max = self.get_init_attr('clip_max', None, neurons)
        neurons.weight_norm_factor = neurons.get_neuron_vec()+self.get_init_attr('norm_factor', 1.0, neurons)

    def new_iteration(self, neurons):
        normalize_synapse_attr('W', 'W', neurons.weight_norm_factor, neurons, self.syn_type)
        for s in neurons.afferent_synapses[self.syn_type]:
            s.W = np.clip(s.W, 0, self.clip_max)






def normalize_synapse_attr(src_attr, target_attr, target_value, neurons, synapse_type):
    neurons.temp_weight_sum = neurons.get_neuron_vec()

    for s in neurons.afferent_synapses[synapse_type]:
        s.dst.temp_weight_sum += np.sum(np.abs(getattr(s, src_attr)), axis=1)

    neurons.temp_weight_sum /= target_value

    for s in neurons.afferent_synapses[synapse_type]:
        setattr(s, target_attr, getattr(s, target_attr) / (s.dst.temp_weight_sum[:, None] + (s.dst.temp_weight_sum[:, None] == 0)))




def normalize_synapse_attr_efferent(src_attr, target_attr, target_value, neurons, synapse_type):
    neurons.temp_weight_sum = neurons.get_neuron_vec()

    for s in neurons.efferent_synapses[synapse_type]:
        s.src.temp_weight_sum += np.sum(np.abs(getattr(s, target_attr)), axis=0)

    neurons.temp_weight_sum /= target_value

    for s in neurons.efferent_synapses[synapse_type]:
        setattr(s, src_attr, getattr(s, src_attr) / (s.src.temp_weight_sum + (s.src.temp_weight_sum == 0)))

'''
from PymoNNto import *

net = Network(tag='Network')

ng = NeuronGroup(tag='Neuron', net=net, behaviour={}, size=10)
sg = SynapseGroup(tag='GLU', src=ng, dst=ng, net=net, behaviour={})

net.initialize()

sg.W = sg.get_synapse_mat('random')

print(np.sum(sg.W,0))
print(np.sum(sg.W,1))

normalize_synapse_attr_efferent('W','W', 1, ng, 'GLU')
normalize_synapse_attr('W','W', 1, ng, 'GLU')

print(np.sum(sg.W,0))
print(np.sum(sg.W,1))

import matplotlib.pyplot as plt

plt.matshow(sg.W)
plt.show()
'''


from PymoNNto.NetworkCore.Base import *
from PymoNNto.Exploration.Evolution.Interface_Functions import *

class Behaviour(NetworkObjectBase):
    set_variables_on_init = False

    def __init__(self, **kwargs):
        self.init_kwargs = kwargs
        self.used_attr_keys = []
        self.behaviour_enabled = self.get_init_attr('behaviour_enabled', True, None)
        super().__init__(tag=self.get_init_attr('tag', None, None))
        self.add_tag(self.__class__.__name__)

    def set_variables(self, neurons):
        return

    def new_iteration(self, neurons):
        return

    def set_gene_variables(self):
        current_genome = {}
        for variable_key in self.init_kwargs:
            while type(self.init_kwargs[variable_key]) is str and '[' in self.init_kwargs[variable_key] and ']' in self.init_kwargs[variable_key]:
                s = self.init_kwargs[variable_key]

                start = s.index('[')
                end = s.index(']')

                content = s[start + 1: end]

                if '#' in content:
                    parts = content.split('#')
                    default_value = float(parts[0])
                    gene_key = parts[1]
                else:
                    gene_key = content
                    default_value = None

                current_genome[gene_key] = get_gene(gene_key, default_value)

                self.init_kwargs[variable_key] = s[:start] + '{:.15f}'.format(current_genome[gene_key]).rstrip('0').rstrip('.') + s[end + 1:]
        return current_genome

    def __str__(self):
        result = self.__class__.__name__+'('
        for k in self.init_kwargs:
            result += str(k) + '=' + str(self.init_kwargs[k])+','
        result += ')'
        return result

    def evaluate_diversity_string(self, ds, neurons_or_synapses):

        if 'same(' in ds and ds[-1] == ')':
            params = ds[5:-1].replace(' ', '').split(',')
            if len(params) == 2:
                return getattr(neurons_or_synapses[params[0], 0], params[1])

        plot = False
        if ';plot' in ds:
            ds = ds.replace(';plot', '')
            plot = True

        result = ds

        if is_number(ds):
            result = float(ds)

        if '%' in ds and is_number(ds.replace('%', '')):
            result = float(ds.replace('%', ''))/100.0

        if '(' in ds and ')' in ds:#is function
            if type(neurons_or_synapses).__name__ == "NeuronGroup":
                result = neurons_or_synapses.get_neuron_vec(ds)

            if type(neurons_or_synapses).__name__ == "SynapseGroup":
                result = neurons_or_synapses.get_synapse_mat(ds)

        if plot:
            if type(result) == np.ndarray:
                import matplotlib.pyplot as plt
                plt.hist(result, bins=30)
                plt.show()

        return result


    def set_init_attrs_as_variables(self, object):
        for key in self.init_kwargs:
            setattr(object, key, self.get_init_attr(key, None, neurons=object))
            print('init', key)
            #get_init_attr

    def check_unused_attrs(self):
        for key in self.init_kwargs:
            if not key in self.used_attr_keys:
                print('Warning: "'+key+'" not used in set_variables of '+str(self)+' behaviour! Make sure that "'+key+'" is spelled correctly and get_init_attr('+key+',...) is called in set_variables. Valid attributes are:'+str(self.used_attr_keys))

    def get_init_attr(self, key, default, neurons=None, do_not_diversify=False, search_other_behaviours=False, required=False):

        if required and not key in self.init_kwargs:
            print('Warning:',key,'has to be specified for the behaviour to run properly.', self)

        self.used_attr_keys.append(key)

        result = self.init_kwargs.get(key, default)

        if key not in self.init_kwargs and neurons is not None and search_other_behaviours:
            for b in neurons.behaviours:
                if key in b.init_kwargs:
                    result = b.init_kwargs.get(key, result)

        if not do_not_diversify and type(result) is str and neurons is not None:
            result = self.evaluate_diversity_string(result, neurons)

        return result



    def visualize_module(self, vmi=None, vmo=None, vma=None):
        from PymoNNto.Exploration.Visualization import Module_visualizer as drawer
        self.visualization_module_inputs = vmi
        self.visualization_module_outputs = vmo
        self.visualization_module_attributes = vma
        md = drawer.module_drawer()
        md.add_module(self)
        md.show()
    
    


from PymoNNto.Exploration.Network_UI.TabBase import *
from PymoNNto.Exploration.Visualization.Module_visualizer import *

class module_visualizer_tab(TabBase):

    def add_recorder_variables(self, neuron_group, Network_UI):
        return

    def initialize(self, Network_UI):

        self.di_modules=[]
        group_tags=[group.tags[0] for group in get_unique_non_partitioned_Groups(Network_UI.network.all_objects())]

        for t,timestep in enumerate(Network_UI.network.behaviour_timesteps):
            #for net_obj in Network_UI.network['exc_neurons']:#Network_UI.network.NeuronGroups:
            for i, net_obj in enumerate(get_unique_non_partitioned_Groups(Network_UI.network.all_objects())):

                if timestep in net_obj.behaviour:# and net_obj.behaviour[timestep].behaviour_enabled
                    beh=net_obj.behaviour[timestep]
                    module_name, inputs, outputs, attributes, module_type = analyze_module_and_get_info(beh)
                    mdi = Module_draw_item(module_name, inputs, outputs, attributes, module_type, x=t*1200, y=i*1200, onlyff=True)
                    self.di_modules.append(mdi)

        self.add_flow_chart(Network_UI, self.di_modules, group_tags, Network_UI.network.behaviour_timesteps)

        Network_UI.Next_H_Block()
        self.connections_cb = Network_UI.Add_element(QCheckBox('connections'))
        def connections_cb_changed():
            for m in self.di_modules:
                m.draw_connection_lines = self.connections_cb.isChecked()
                m.update_pic(self.di_modules)
        self.connections_cb.setChecked(True)
        self.connections_cb.stateChanged.connect(connections_cb_changed)

        self.in_out_cb = Network_UI.Add_element(QCheckBox('inputs/outputs'))
        def in_out_cb_changed():
            for m in self.di_modules:
                m.draw_in_out_blocks = self.in_out_cb.isChecked()
                m.update_pic(self.di_modules)
        self.in_out_cb.setChecked(True)
        self.in_out_cb.stateChanged.connect(in_out_cb_changed)

        self.attributes_cb = Network_UI.Add_element(QCheckBox('attributes'))
        def attributes_cb_changed():
            for m in self.di_modules:
                m.draw_attributes = self.attributes_cb.isChecked()
                m.update_pic(self.di_modules)
        self.attributes_cb.setChecked(True)
        self.attributes_cb.stateChanged.connect(attributes_cb_changed)


        self.hlines_cb = Network_UI.Add_element(QCheckBox('hlines'))
        def h_line_cb_changed():
            for l in self.h_line_list:
                if not self.hlines_cb.isChecked():
                    self.flow_tab.plot.removeItem(l)
                else:
                    self.flow_tab.plot.addItem(l)
        self.hlines_cb.setChecked(True)
        self.hlines_cb.stateChanged.connect(h_line_cb_changed)

        self.vlines_cb = Network_UI.Add_element(QCheckBox('vlines'))
        def v_line_cb_changed():
            for l in self.v_line_list:
                if not self.vlines_cb.isChecked():
                    self.flow_tab.plot.removeItem(l)
                else:
                    self.flow_tab.plot.addItem(l)
        self.vlines_cb.setChecked(True)
        self.vlines_cb.stateChanged.connect(v_line_cb_changed)

        self.keys_cb = Network_UI.Add_element(QCheckBox('keys'))
        def keys_cb_changed():
            for k in self.key_text_list:
                if not self.keys_cb.isChecked():
                    self.flow_tab.plot.removeItem(k)
                else:
                    self.flow_tab.plot.addItem(k)
        self.keys_cb.setChecked(True)
        self.keys_cb.stateChanged.connect(keys_cb_changed)

    def update(self, Network_UI):
        return

    def add_flow_chart(self, Network_UI, di_modules, group_tags, keys):
        self.flow_tab = Network_UI.Next_Tab('Flow_Chart', stretch=0.0)
        self.flow_tab.plot = Network_UI.Add_plot()
        self.flow_tab.plot.hideAxis('bottom')
        self.flow_tab.plot.hideAxis('left')
        self.flow_tab.modules = []

        def on_plot_click_event(event):
            for mod in self.flow_tab.modules:
                mod.selected = True

            for mod in self.flow_tab.modules:
                mod.update_pic(self.flow_tab.modules)

            self.flow_tab.plot.update()

        self.flow_tab.plot.mouseClickEvent = on_plot_click_event

        self.v_line_list = []
        self.key_text_list = []
        for i, k_s in enumerate(keys):
            text = pg.TextItem(text=str(k_s), anchor=(0, 0), color=(0,0,0))
            text.setPos(1200*i+300, 600-len(group_tags)*1200)
            self.key_text_list.append(text)
            self.flow_tab.plot.addItem(text)
            line = pg.InfiniteLine(-100 + i * 1200, 90, pen=(200,200,200))
            self.v_line_list.append(line)
            self.flow_tab.plot.addItem(line)

        self.h_line_list = []
        for i, g_s in enumerate(group_tags):
            text = pg.TextItem(text=g_s, anchor=(0, 0), color=(0,0,0))
            text.setPos(-1200*2, 600-i*1200)
            self.flow_tab.plot.addItem(text)
            line = pg.InfiniteLine(-100-i*1200, 0, pen=(100,100,100))
            self.h_line_list.append(line)
            self.flow_tab.plot.addItem(line)


        for mod in di_modules:
            self.flow_tab.modules.append(mod)
            self.flow_tab.plot.addItem(mod)

            def on_mod_click_event(event):
                for mod in self.flow_tab.modules:
                    mod.selected = False

                event.currentItem.selected = True

                for mod in self.flow_tab.modules:
                    mod.update_pic(self.flow_tab.modules)
                self.flow_tab.plot.update()

            mod.mouseClickEvent = on_mod_click_event

        for mod in self.flow_tab.modules:
            mod.update_pic(self.flow_tab.modules)
        self.flow_tab.plot.update()


from PymoNNto.NetworkCore.Base import *
import inspect

#This class can be used to add tag-searchable functions to the neurongroups, synapsegroups and the network object.
#It has a main execute funtion which can be called with module(...) or module.exec(...)
#Other "normal" functions can be added as well.
#Via the add_tag function, the modules can be categorized into groups

class AnalysisModule(NetworkObjectBase):

    def __init__(self, parent=None, **kwargs):
        self.init_kwargs = kwargs
        super().__init__(tag=self.get_init_attr('tag', None))

        self.used_attr_keys = []

        self.add_tag(self.__class__.__name__)

        self.execution_arguments = self._get_default_args_(self.execute)

        self.result_storage = {}#key=arguments value=result from run
        self.current_result = None
        self.save_results = True

        self.update_notifier_functions = []

        if parent is not None:
            self._attach_and_initialize_(parent)

        self.progress_update_function=None

    def add_progress_update_function(self, function):
        self.progress_update_function = function

    def update_progress(self, percent):
        if self.progress_update_function is not None:
            self.progress_update_function(percent)

    def _attach_and_initialize_(self, parent):
        self.parent = parent  # NeuronGroup SynapseGroup or Network
        parent.analysis_modules.append(self)
        setattr(parent, self.__class__.__name__, self) #ng1.AnalysisModule(...)
        self.initialize(parent)

    def initialize(self, neurons): #override
        #access arguments via get_init_attr(key, default)
        #self.add_tag('classifier')
        #self.add_execution_argument(...)
        #execute does not have to be used. You can just add all kinds of easy access convenience funtions to the class
        return

    def execute(self, neurons, **kwargs): #override (do not call directly with instance.execute(...). Use instance(...))
        #self.parent
        return


    def is_executable(self):
        return type(self).execute != AnalysisModule.execute
        #return self.execute is super(AnalysisModule, self).execute

    def get_init_attr(self, key, default):
        if key in self.init_kwargs:
            return self.init_kwargs[key]
        else:
            return default

    def _update_notification_(self, key=None):
        for function in self.update_notifier_functions:
            function(key)

    def remove_update_notifier(self, function):
        if function in self.update_notifier_functions:
            self.update_notifier_functions.remove(function)

    def set_update_notifier(self, function):
        self.update_notifier_functions.append(function)

    #def add_execution_argument(self, key, default):
    #    self.execution_arguments[key] = default

    def __call__(self, **kwargs):
        self.update_progress(0)
        self.current_key = self.generate_current_key(kwargs)
        return self.save_result(self.current_key,self.execute(self.parent, **kwargs))

    def exec(self, **kwargs):
        self.update_progress(0)
        self.current_key = self.generate_current_key(kwargs)
        return self.save_result(self.current_key,self.execute(self.parent, **kwargs))


    def _get_base_name_(self):
        return self.__class__.__name__

    def generate_current_key(self, args_key, add_args=True):
        key = self._get_base_name_()
        if len(args_key) > 0 and add_args:
            key += '(' + str(args_key) + ')'
        return key

    def save_result(self, key, result):
        self.update_progress(100)
        if self.save_results and result is not None:
            self.current_result = result
            self.result_storage[key] = result
        self._update_notification_(key)
        return result

    def last_call_result(self):
        return self.current_result

    def get_results(self):
        return self.result_storage

    def remove_result(self, key):
        if key in self.result_storage:
            return self.result_storage.pop(key)
        else:
            print('cannot remove result', key, 'not found.')
        self._update_notification_()

    def clear_results(self):
        self.result_storage = {}
        self._update_notification_()

    def _get_default_args_(self, func, exclude=['self', 'args', 'kwargs'], exclude_first=True):
        result = {}
        signature = inspect.signature(func)
        i=0
        for k, v in signature.parameters.items():
            if i>0 or not exclude_first:
                if k not in exclude:
                    if v.default is not inspect.Parameter.empty:
                        result[k] = v.default
                    else:
                        result[k] = ''
            i += 1
        return result





from PymoNNto.Exploration.Evolution.Devices.Evolution_Device_Multi_Thread import *
import sys
from PymoNNto.Exploration.Evolution.SSH_Functions import *

def ssh_thread_worker(slave_file, name, user, host, password, python_cmd, root_folder, conn):
    print('ssh thread started')
    while True:
        time.sleep(np.random.rand())
        if conn.poll():
            genome = conn.recv()
            try:
                ssh = get_ssh_connection(host, user, password)
                cmd = 'cd ' + name + '/' +root_folder+ '; '
                cmd += python_cmd+' ' + slave_file + ' genome=' + get_gene_id(genome)
                ssh_stdin, ssh_stdout, ssh_stderr = ssh.exec_command(cmd)
                results = get_response(ssh_stdout, ssh_stderr)
                ssh.close()

                success = False
                for line in results:
                    if 'evolution score set to #' in line:
                        parts = line.split('#')
                        if len(parts) == 3:
                            genome['score'] = float(parts[1])
                            success = True

                if success:
                    sm = StorageManager(main_folder_name=genome['evo_name'], folder_name=get_gene_file(genome), print_msg=False, add_new_when_exists=False)
                    sm.save_param_dict(genome)
                    conn.send([genome, 'success'])
                else:
                    conn.send([genome, 'no score found in ssh thread results'+str(results)])

            except:
                conn.send([genome, 'thread error'])

        conn.send([None, 'idle'])
        time.sleep(1.0)


class Evolution_Device_SSH(Evolution_Device_Multi_Thread):

    def __init__(self, device_string, parent):
        super().__init__(device_string, parent)

        self.python_cmd = 'python3'

        for s in self.device_string.split(' '):
            if 'python_cmd=' in s:
                self.python_cmd = s.split('=')[1].replace('"','').replace("'",'')


        #self.user = None
        #self.host = None
        #self.password = None

        temp=sys.argv[0].replace(get_data_folder().replace('Data', ''), '')
        temp='/'.join(temp.split('/')[:-1])#remove file name
        self.root_folder = temp #evolution_master_path_from_project_folder

        self.user, self.host, self.password = split_ssh_user_host_password_string(self.device_string)

    def initialize(self):
        self.parent_conn, child_conn = Pipe()
        self.process = Process(target=ssh_thread_worker, args=(self.parent.slave_file, self.parent.name, self.user, self.host, self.password, self.python_cmd, self.root_folder, child_conn))

    def initialize_device_group(self):
        transfer_project(self.parent.name, self.user, self.host, self.password)



import numpy as np
import matplotlib.pyplot as plt

#average_act = np.mean(np.array(e_ng['n.output'][0]), axis=1)

def inhibition_excitation_scatter(inh, exc):
    inh = np.mean(inh, axis=0)
    exc = np.mean(exc, axis=0)

    plt.scatter(exc, inh)
    plt.xlabel('excitation')
    plt.ylabel('inhibition')
    plt.show()

def get_t_vs_tp1_mat(average_activities_x, average_activities_y, resolution=100, flip=True, no_act_compensation=True):
    heatmap = np.zeros((resolution, resolution))
    max_act = np.maximum(np.max(average_activities_x), np.max(average_activities_y))

    if max_act > 0:
        for i in range(np.minimum(len(average_activities_x)-1, len(average_activities_y)-1)):
            x = int(np.trunc((resolution-1)/max_act*average_activities_x[i]))
            y = int(np.trunc((resolution-1)/max_act*average_activities_y[i+1]))
            heatmap[y, x] += 1

    if no_act_compensation:
        heatmap[0, 0] = 0

    if flip:
        heatmap = np.flip(heatmap, axis=0)

    return heatmap

def plot_t_vs_tp1(average_activities, resolution=100):
    heatmap=get_t_vs_tp1_mat(average_activities,average_activities, resolution)
    plt.imshow(heatmap)
    plt.xlabel('t')
    plt.ylabel('t+1')
    plt.xticks([])
    plt.yticks([])
    #plt.axis('off')
    plt.show()



from PymoNNto.NetworkCore.Behaviour import *
from PymoNNto.NetworkCore.Synapse_Group import *

class Partition(Behaviour):
    visualization_module_inputs = ['s.src.x', 's.src.y', 's.src.z', 's.dst.x', 's.dst.y', 's.dst.z']
    visualization_module_outputs = ['s.network.SynapseGroups']

    set_variables_on_init = True

    def partition(self, synapses, dst_group_masks):  # todo:auto receptive field extraction (blocks dont need to be squared!)

        rf_x, rf_y, rf_z = synapses.get_max_receptive_field_size()

        syn_sub_groups = []

        for dst_mask in dst_group_masks:

            dst_subgroup = synapses.dst.subGroup(dst_mask)

            src_mask = synapses.src.get_subgroup_receptive_field_mask(dst_subgroup, [rf_x, rf_y, rf_z])

            #src_x_start = np.min(dst_subgroup.x) - rf_x
            #src_x_end = np.max(dst_subgroup.x) + rf_x

            #src_y_start = np.min(dst_subgroup.y) - rf_y
            #src_y_end = np.max(dst_subgroup.y) + rf_y

            #src_z_start = np.min(dst_subgroup.z) - rf_z
            #src_z_end = np.max(dst_subgroup.z) + rf_z

            #src_mask = (synapses.src.x >= src_x_start) * (synapses.src.x <= src_x_end) * (synapses.src.y >= src_y_start) * (synapses.src.y <= src_y_end) * (synapses.src.z >= src_z_start) * (synapses.src.z <= src_z_end)

            syn_sub_groups.append(synapses.get_sub_synapse_group(src_mask, dst_mask))

        print('partitioned into', len(syn_sub_groups), 'SynapseGroups')

        # add sub Groups
        for sg in syn_sub_groups:
            sg.tags.append('partitioned')
            synapses.network.SynapseGroups.append(sg)

        # remove original SG
        synapses.network.SynapseGroups.remove(synapses)

        return syn_sub_groups


    def set_variables(self, synapses):

        split_size = self.get_init_attr('split_size', 'auto')

        if split_size == 'auto':
            best_block_size = 7
            w = int((synapses.src.width/best_block_size+synapses.dst.width/best_block_size)/2)
            h = int((synapses.src.height/best_block_size+synapses.dst.height/best_block_size)/2)
            d = int((synapses.src.depth / best_block_size + synapses.dst.depth / best_block_size) / 2)
            split_size = [np.maximum(w, 1), np.maximum(h, 1), np.maximum(d, 1)]
            if split_size[0]<3 and split_size[1]<3 and split_size[2]<3:
                return #do not partition in this case

        #split_size = synapses.dst.partition_size()
        dst_sub_masks = synapses.dst.partition_masks(split_size)
        self.partition(synapses, dst_sub_masks)#this removes "synapses" from network and adds new SynapseGroups!


    def new_iteration(self, synapses):
        return

#p=Partition()
#p.visualize_module()


from PymoNNto.Exploration.Evolution.Devices.Evolution_Device import *
from multiprocessing import Process, Pipe
#import subprocess
#import time
#import os
import numpy as np
import sys


def local_thread_worker(slave_file, conn):
    #try:
    #    import psutil as psu
    #    parent = psu.Process()
    #    parent.nice(psu.BELOW_NORMAL_PRIORITY_CLASS)
    #except:
    #    print('not able to set BELOW_NORMAL_PRIORITY_CLASS')
    print('local thread started')
    while True:
        time.sleep(np.random.rand()/10)
        if conn.poll():
            genome = conn.recv()
            try:
                execute_local_file(slave_file, genome)
                conn.send([genome, 'success'])
            except Exception as e:
                error_type = str(sys.exc_info()[0])
                msg = str(sys.exc_info()[1])
                conn.send([genome, 'thread error '+error_type+' '+msg])

        conn.send([None, 'idle'])


class Evolution_Device_Multi_Thread(Evolution_Device):

    def get_score(self, genome):
        sm = StorageManager(main_folder_name=genome['evo_name'], folder_name=get_gene_file(genome), add_new_when_exists=False)#, print_msg=False
        return sm.load_param('score', default=None)

    def initialize(self):
        self.parent_conn, child_conn = Pipe()
        self.process = Process(target=local_thread_worker, args=(self.parent.slave_file, child_conn))

    def start(self):
        self.process.start()

    def main_loop_update(self):

        if self.parent_conn.poll():
            genome, thread_msg = self.parent_conn.recv()

            #print(thread_msg)

            if thread_msg == 'success':
                self.score_processing(genome)

            elif thread_msg == 'idle':
                current_genome = self.parent.get_next_genome()
                if current_genome is not None:
                    self.parent_conn.send(current_genome)
                #else:
                #    print('no genes found to process')

            else:
                self.error_event(genome, thread_msg)


    def stop(self):
        return
        #self.process.stop()#???





# WiltingPriesemann toolbox for MR estimation
# Authors: Jens Wilting & Viola Priesemann
# Max-Planck-Institute for Dynamics and Self-Organization
# Am Fassberg 17
# D-37077 Goettingen
# Germany
#
# Minimal toolbox for MR estimation according to
# Wilting & Priesemann, 2018

import numpy as np
import scipy.stats
import scipy.interpolate


# ============================================================================
# Simulation of branching processes with immigration
# Ref: C. Heathcote, J R Stat Soc Ser B, 1965
# All variable names according to Wilting & Priesemann, 2018
# ============================================================================
def simulate_branching(length=10000, m=0.9, activity=100):
    
    h = activity * (1 - m)
    
    A_t = np.zeros(length, dtype=int)
    A_t[0] = np.random.poisson(lam=activity)
    
    print('Generating branching process with ' + str(length )+ ' time steps, m=' + str(m) + ', and drive rate' + str(h))
    
    for idx in range(1, length):
        if not idx % 1000:
            print(str(idx) + " loops completed")
        tmp = 0
        tmp += np.random.poisson(lam=h)
        if m > 0:
            #for idx2 in range(A_t[idx - 1]):
            tmp += np.random.poisson(lam=m, size=A_t[idx - 1]).sum()      
            
        A_t[idx] = tmp
        
    print("Branching process created with mean activity " + str(A_t.mean()))

    return A_t



# ============================================================================
# Emulation of binomial subsampling for given time series
# Ref: Wilting & Priesemann, 2018
# All variable names according to Wilting & Priesemann, 2018
# ============================================================================
def simulate_binomial_subsampling(A_t, alpha):
    a_t = np.zeros(len(A_t))
    for idx, item in enumerate(A_t):
        a_t[idx] = scipy.stats.binom.rvs(A_t[idx], alpha)

    return a_t



# ============================================================================
# Helper routine to handle different input formats
# for possible input formats, see below
# ============================================================================
def input_handler(items):
    situation = -1
    if isinstance(items, np.ndarray):
        if items.dtype.kind == 'i' or items.dtype.kind == 'f' or items.dtype.kind == 'u' :
            items = [items]
            situation = 0
        elif items.dtype.kind == 'S':
            situation = 1
        else:
            raise Exception('Invalid input: numpy.ndarray which is neither data nor file name.')
    elif isinstance(items, list):
        if all(isinstance(item,str) for item in items):
            situation = 1
        elif all(isinstance(item,np.ndarray) for item in items):
            situation = 0
        else:
            raise Exception('Invalid input: input type not recognized')
    elif isinstance(items, basestring):
        items = [items]
        situation = 1
    else:
        raise Exception('Invalid input: input type not recognized')
    

    if situation == 0:
        return items
    elif situation == 1:
        data = []
        for idx, item in enumerate(items):
            data.append(np.load(item))
        return data
    else:
        raise Exception('Unknown situation!')
    
    
    
# ============================================================================
# Helper routine to compute linear regression slopes $r_k$
# ============================================================================
def get_slopes(all_counts, k_min, k_max, scatterpoints):
        
    k = np.arange(k_min, k_max)#original:1
    r_k = np.zeros(len(k))
    intercepts = np.zeros(len(k))
    r_values = np.zeros(len(k))
    p_values = np.zeros(len(k))
    std_errs = np.zeros(len(k))

    xs = []
    ys = []

    #print(all_counts[0].shape)

    for idx, step in enumerate(k):
        x = np.empty(0)
        y = np.empty(0)
        for counts in all_counts:
            x = np.concatenate((x, counts[0:-step]))
            y = np.concatenate((y, counts[step:]))
            
        slope, intercept, r_value, p_value, std_err = scipy.stats.linregress(x,y)
        r_k[idx] = slope
        intercepts[idx] = intercept
        r_values[idx] = r_value
        p_values[idx] = p_value
        std_errs[idx] = std_err
        
        if idx == 0:
            data_length = len(x) + 1
            mean_activity = x.mean()
        
        if scatterpoints:
            if scatterpoints == True:
                xs.append(x)
                ys.append(y)  
            else:
                x = x[0::len(x) / scatterpoints]
                y = y[0::len(y) / scatterpoints]
                xs.append(x)
                ys.append(y)
                
    return k, r_k, intercepts, r_values, p_values, std_errs, data_length, mean_activity, xs, ys



# ============================================================================
# Helper routine for exponential model
# ============================================================================
def func(x, a, b, string=False):
    if string:
        return (r'$r_k = ' + str(round(a, 3))
                + '\cdot' + str(round(b, 3)) + '^k$')
    else:
        return np.abs(a)*np.abs(b)**x
    
    
    
# ============================================================================
# MR estimation routine
# Ref: Wilting & Priesemann, 2018
# All variable names according to Wilting & Priesemann, 2018
#
# Required input: time series (see below for possible formats), k_max
#
# Returns: dict with information from MR estimation
#
# Possible data formats (will be expanded in the future):
# 1) default time series as numpy.array
# 2) multiple time series as list [numpy.array, numpy.array, ...]
# 3) path to a .npy file with time series
# 4) list of paths to multiple .npy files with time series
# ============================================================================
def MR_estimation(all_counts, minslopes=1, maxslopes=40, scatterpoints=False, fractions=1):
    
    # scatterpoints: if scatterpoints of all regressions shall be returned. Can consume a lot of memory if True

    #print(np.array(all_counts).shape)

    return_dict = dict()
    
    all_counts = np.array(input_handler(all_counts))

    fraction_block_size=int(np.floor(len(all_counts[0])/fractions))

    k = []
    r_k = []
    #intercepts = []
    #r_values = []
    #p_values = []
    std_errs = []
    data_length = []
    #mean_activity = []
    #xs = []
    #ys = []

    for f in range(fractions):
        spike_fraction = all_counts[:, fraction_block_size*f:fraction_block_size*(f+1)]
        #print(f, len(spike_fraction[0]))
        # Get all slopes r_k
        _k, _r_k, _intercepts, _r_values, _p_values, _std_errs, _data_length, _mean_activity, _xs, _ys = get_slopes(spike_fraction, minslopes, maxslopes, scatterpoints)

        k.append(_k)
        r_k.append(_r_k)
        #intercepts.append(_intercepts)
        #r_values.append(_r_values)
        #p_values.append(_p_values)
        std_errs.append(_std_errs)
        data_length.append(_data_length)
        #mean_activity.append(_mean_activity)
        #xs.append(_xs)
        #ys.append(_ys)

    k = k[0]
    r_k = np.mean(r_k, axis=0)
    #intercepts=intercepts[0]
    #r_values=np.mean(r_values, axis=0)
    #p_values=np.mean(p_values, axis=0)
    std_errs=np.mean(std_errs, axis=0)
    data_length=data_length[0]
    #mean_activity=np.mean(mean_activity, axis=0)
    #xs=xs[0]
    #ys=ys[0]

    
    # Fit m, b (here as p_opt = [b, m]) according to exponential model        
    fitfunc = func
    p0 = [r_k[0], 1.0]

    p_opt, pcov = scipy.optimize.curve_fit(fitfunc, k, r_k, p0=p0, maxfev=100000, sigma=std_errs * np.linspace(1, 10, len(std_errs)))

    return_dict['branching_ratio'] = p_opt[1]
    return_dict['autocorrelationtime'] = - 1.0 / np.log(p_opt[1])
    return_dict['naive_branching_ratio'] = r_k[0]
    return_dict['k'] = k
    return_dict['r_k'] = r_k
    return_dict['fitfunc'] = fitfunc
    #if scatterpoints:
    #    if onlytwoscatterpoints:
    #        return_dict['xs'] = [xs[0], xs[-1]]
    #        return_dict['ys'] = [ys[0], ys[-1]]
    #    else:
    #        return_dict['xs'] = xs
    #        return_dict['ys'] = ys
    return_dict['data_length'] = data_length
    return_dict['p_opt'] = p_opt

    return return_dict

#import matplotlib.pyplot as plt
#act=np.random.rand(10000)
#data = MR_estimation(act, 1, 100, fractions=4)
#plt.scatter(data['k'], data['r_k'])
#plt.show()


#import sys
#sys.path.append('../../../TREN2')
#sys.path.append('../../../tren2')

#from X_Experimental.Functions import *

import matplotlib.pylab as plt
import matplotlib as mpl

mpl.rcParams["savefig.directory"] = '~/Desktop/'

from matplotlib.collections import PolyCollection
import matplotlib.pyplot as plt
from matplotlib import colors as mcolors
import configparser
from scipy.stats import entropy


def wrong_new_rigt_bar_comparison(tag, x,h,w,l,e,c):

    smg = StorageManagerGroup(tag)
    #smg.sort_by('score')
    #axis = smg.get_multi_param_list(['score', ''])

    def add_bar(lab, val, err, col):
        x.append(x[-1]+1)
        h.append(val)
        w.append(1.0)
        l.append(lab)
        e.append(err)#std
        c.append(col)
        return x[-1], val

    wrong_percentages = []
    right_percentages = []
    new_percentages = []

    for sm in smg.StorageManagerList:
        n_output_sentences = sm.load_param('n_output_sentences')
        n_wrong_sentences = sm.load_param('n_wrong_sentences')
        n_new_sentences = sm.load_param('n_new_sentences')
        n_right_sentences = sm.load_param('n_right_sentences')

        if n_output_sentences is not None and n_wrong_sentences is not None and n_new_sentences is not None and n_right_sentences is not None:
            wrong_percentages.append(n_wrong_sentences / n_output_sentences)
            right_percentages.append(n_new_sentences / n_output_sentences)
            new_percentages.append(n_right_sentences / n_output_sentences)

    _, yp1 = add_bar('w', np.average(wrong_percentages), np.std(wrong_percentages), 'red')
    xp, yp2 = add_bar('n', np.average(right_percentages), np.std(right_percentages), 'yellow')
    _, yp3 = add_bar('r', np.average(new_percentages), np.std(new_percentages), 'green')

    #plt.text(xp, np.max([wrong_percentages, right_percentages, new_percentages])+0.001, tag.replace('_layer_1l_exc_act', '').replace('_inpd', ''), fontsize=12, horizontalalignment='center', rotation=90)


def plot_wrong_new_right_bar(tags=[]):

    x = [0]
    h = [0]
    w = [0]
    l = ['']
    e = [0]
    c = ['white']

    for i, t in enumerate(tags):
        wrong_new_rigt_bar_comparison(t, x, h, w, l, e, c)

        for _ in range(2):
            x.append(x[-1]+1)
            h.append(0)
            w.append(0)
            l.append('')
            e.append(0)
            c.append('white')

    plt.bar(x, h, w, tick_label=l, yerr=e, color=c)

    plt.show()

def plot_wrong_new_right_graph(tags=[]):

    x = []
    w = []
    ws = []
    r = []
    rs = []
    n = []
    ns = []

    for i, t in enumerate(tags):
        wrong_percentages = []
        right_percentages = []
        new_percentages = []

        smg = StorageManagerGroup(t)

        for sm in smg.StorageManagerList:
            n_output_sentences = sm.load_param('n_output_sentences')
            n_wrong_sentences = sm.load_param('n_wrong_sentences')
            n_new_sentences = sm.load_param('n_new_sentences')
            n_right_sentences = sm.load_param('n_right_sentences')

            if n_output_sentences is not None and n_wrong_sentences is not None and n_new_sentences is not None and n_right_sentences is not None:
                wrong_percentages.append(n_wrong_sentences / n_output_sentences)
                right_percentages.append(n_new_sentences / n_output_sentences)
                new_percentages.append(n_right_sentences / n_output_sentences)

        x.append(i+1)
        w.append(np.average(wrong_percentages))
        ws.append(np.std(wrong_percentages))

        r.append(np.average(right_percentages))
        rs.append(np.std(right_percentages))

        n.append(np.average(new_percentages))
        ns.append(np.std(new_percentages))

        #_, yp1 = add_bar('w', np.average(wrong_percentages), np.std(wrong_percentages), 'red')
        #xp, yp2 = add_bar('n', np.average(right_percentages), np.std(right_percentages), 'yellow')
        #_, yp3 = add_bar('r', np.average(new_percentages), np.std(new_percentages), 'green')

    x=np.array(x)*10000

    plt.plot(x, w, label='wrong')
    plt.fill_between(x, np.array(w) - np.array(ws), np.array(w) + np.array(ws), alpha=0.2)
    plt.plot(x, r, label='right')
    plt.fill_between(x, np.array(r) - np.array(rs), np.array(r) + np.array(rs), alpha=0.2)
    plt.plot(x, n, label='new')
    plt.fill_between(x, np.array(n) - np.array(ns), np.array(n) + np.array(ns), alpha=0.2)

    plt.legend(loc='best', frameon=False, fontsize=20)

    plt.show()


def comp_plot(tags=[]):

    x = [0]
    h = [0]
    w = [0]
    l = ['']
    e = [0]
    c = ['white']

    sv = []
    xsv = []

    fig, axs = plt.subplots(4)
    fig.suptitle('Learning performance - Singular values - Wilting-Priesmann r values - Average activity')

    train_s = 5000
    eval_s = 1000

    #component_image = np.zeros((100,900))

    for i, t in enumerate(tags):
        wrong_new_rigt_bar_comparison(t, x, h, w, l, e, c)

        for _ in range(2):
            x.append(x[-1]+1)
            h.append(0)
            w.append(0)
            l.append('')
            e.append(0)
            c.append('white')

        smg = StorageManagerGroup(t)

        #.load_param('n_output_sentences')

        singular_values = smg.StorageManagerList[0].load_np('singluar_values')
        sv.append(singular_values)
        xsv.append((i+1)*(train_s+eval_s))




        r_k = smg.StorageManagerList[0].load_np('r_k')
        x_p=np.arange(len(r_k))*30+i*(train_s+eval_s)
        axs[2].scatter(x_p, r_k, s=1)

        axs[2].set_xticklabels([])

        #axs[2].set_xticks(x, np.arange(len(x)))


        mean_act = smg.StorageManagerList[0].load_np('act_exc')
        axs[3].plot(np.arange(len(mean_act))+i*(train_s+eval_s), mean_act, linewidth=1)

        axs[1].axvline(i * (train_s + eval_s) + train_s)
        axs[1].axvline(i * (train_s + eval_s) + train_s + eval_s)

        axs[3].axvline(i * (train_s + eval_s) + train_s)
        axs[3].axvline(i * (train_s + eval_s) + train_s + eval_s)


        axs[1].axvspan(i * (train_s + eval_s), i * (train_s + eval_s) + train_s, facecolor='red', alpha=0.1)
        axs[1].axvspan(i * (train_s + eval_s), i * (train_s + eval_s) + train_s + eval_s, facecolor='green', alpha=0.1)

        axs[3].axvspan(i * (train_s + eval_s), i * (train_s + eval_s) + train_s, facecolor='red', alpha=0.1)
        axs[3].axvspan(i * (train_s + eval_s), i * (train_s + eval_s) + train_s + eval_s, facecolor='green', alpha=0.1)

        #for sm in smg.StorageManagerList:
        #    n_output_sentences = sm.load_param('n_output_sentences')


        components = smg.StorageManagerList[0].load_np('components')

        #component_image=np.concatenate([component_image, np.zeros((100, 900)), np.abs(components)*10], axis=0)



    axs[0].bar(x, h, w, tick_label=l, yerr=e, color=c)
    axs[1].plot(xsv, sv)
    axs[1].scatter([0],[0], s=0)

    axs[0].set_xlabel('wrong new right sentences')
    axs[0].set_ylabel('share')

    axs[1].set_xlabel('timesteps')
    axs[1].set_ylabel('singular values')

    axs[2].set_xlabel('delta t (1-150 blocks)')
    axs[2].set_ylabel('correlation coefficient (wilting priesmann)')

    axs[3].set_xlabel('timesteps')
    axs[3].set_ylabel('average excitatory neuron activity')

    #axs[3].plot(xsv, sv)

    #plt.bar(x, h, w, tick_label=l, yerr=e, color=c)

    plt.show()

    plt.imshow(components.transpose(), cmap=plt.get_cmap('gray'), interpolation='nearest', aspect='auto')

    plt.xlabel('component vectors')
    plt.ylabel('neurons')

    plt.show()



if __name__ == '__main__':

    from PymoNNto.Exploration.StorageManager.Storage_Manager_Dir_Select_Dialog import *
    plot_wrong_new_right_bar(tags=Storage_Manager_Dir_Select_Dialog())
    #plot_wrong_new_right_graph(tags=Storage_Manager_Dir_Select_Dialog())
    #comp_plot(tags=Storage_Manager_Dir_Select_Dialog())



def plot_netsize_vs_score():
    lookup_dict = {'n_wrong_sentences': 'wrong', 'IP_Test_diffuse': 'diffuse', 'n_new': 'new', 'n_wrong': 'wrong',
                   'folder_number': 'id', 'IP_Test_New': 'IP', 'IP_Test_New_and_diffuse': 'IP+diffuse'}

    def lookup(key):
        if key in lookup_dict:
            return lookup_dict[key]
        else:
            return key

    # Tags = ['SORN_One_layer', 'SORN_Two_layer', 'SORN_Three_layer']
    # Tags = ['SORN_One_layer', 'Adj_SORN_Two_layer', 'Adj_SORN_Three_layer']
    #Tags = ['SORN_Gram_train_time_50k', 'SORN_Gram_train_time_100k', 'SORN_Gram_train_time_200k',
    #        'SORN_Gram_train_time_300k']

    Tags = ['size_comp', 'bruno_size_comp', 'hierarchical_size_comp']

    params = ['N_e', 'n_output', 'n_wrong_sentences']  # , 'n_new' #, 'n_wrong_sentences'  #'n_wrong_sentences'

    print(params[1:])

    for tag in Tags:
        smg = StorageManagerGroup(tag)

        remove=[]
        for sm in smg.StorageManagerList:
            out=sm.load_param('n_output')
            wrong=sm.load_param('n_wrong_sentences')
            if out is None or wrong is None or 100/out*wrong>95:
                remove.append(sm)
        smg.StorageManagerList=[sm for sm in smg.StorageManagerList if sm not in remove]

        '''
        scores_old = []
        scores_new = []
        for sm in smg.StorageManagerList:
            source = FDTGrammarActivator_New(tag='grammar_act', output_size=sm.load_param('N_e'), random_blocks=True)

            text=sm.load_param('output')
            if text is not None:
                scores_old.append(source.get_text_score_old(text)['n_wrong'])
                scores_new.append(source.get_text_score(text)['n_wrong_sentences'])

        print(scores_old)
        print(scores_new)
        '''

        smg.sort_by(params[0])
        axis = smg.get_multi_param_list(params)  # axis[0]=N_e vec, axis[1]=n_output vec, ...
        #print(axis)

        # convert from number to percentage
        axis[2] = 100 / axis[1] * axis[2]
        # axis[3] = 100 / axis[1] * axis[3]

        for a, p in zip(axis[2:], params[2:]):  # remove first list (N_e)

            x, avg = smg.remove_duplicates_get_eval(axis[0], a, 'np.average(a)')
            x, std = smg.remove_duplicates_get_eval(axis[0], a, 'np.std(a)')

            #print(x, avg)

            plt.plot(x, avg, label=lookup(p) + ' ' + lookup(tag))
            plt.fill_between(x, avg - std, avg + std, alpha=0.2)

            plt.scatter(axis[0], a)

    # plt.axhline(1/540)

    fig_lettersize = 20
    plt.legend(loc='best', frameon=False, fontsize=fig_lettersize)
    plt.xlabel(lookup(params[0]), fontsize=fig_lettersize)
    plt.ylabel(r'sentences (%)', fontsize=fig_lettersize)

    if True:
        format = 'png'
        plot_sm = StorageManager('Plots', '_'.join(Tags) + '___' + '_'.join(params))
        plot_sm.save_param('Tags', str(Tags))
        plot_sm.save_param('params', str(params))
        plot_sm.save_param('lookup', str(lookup))
        plot_sm.save_param('fig_lettersize', fig_lettersize)
        plt.savefig(plot_sm.absolute_path + 'plot.' + format, format=format)

    plt.show()


def plot_netsize_vs_freq_spont_music(res_path, exp_sup_name, n_gram=1, remove_infrequent_ones=False, fig_path=None, title=None):
    first = True
    perf = []
    n = []
    for i in os.listdir(res_path):
        if i.startswith(exp_sup_name+'_'):
            perf.append(pickle.load(open(os.path.join(res_path,i, 'score_spontaneous.obj'), 'rb')))
            config = configparser.ConfigParser()
            config.read(os.path.join(res_path,i,'config.ini'))
            n.append(config['Parameters']['n_e'])

    if remove_infrequent_ones: 
        # clean the dictionaries such that they are the same (we delete occurences that have frequency below 10e-5)
        for i in range(len(perf)):
            for k, v in list(perf[i]['input_frequencies_{}'.format(n_gram)].items()):
                if v < 10e-4:
                    del perf[i]['input_frequencies_{}'.format(n_gram)][k]
                    del perf[i]['output_frequencies_{}'.format(n_gram)][k]
                    perf[i]['sorted_symbols_{}'.format(n_gram)].remove(k)
    # sort them
    n_int = [int(x) for x in n]
    indices = np.argsort(n_int)
    perf = [perf[x] for x in indices]
    n = [n[x] for x in indices]

    for i in range(len(perf)):
        if first: 
            freq_input = perf[i]['input_frequencies_{}'.format(n_gram)]
            sorted_notes = perf[i]['sorted_symbols_{}'.format(n_gram)]
            first = False
        freq_output = perf[i]['output_frequencies_{}'.format(n_gram)]
        assert list(freq_output.keys()) == list(freq_input.keys()), "input and output dictionaries are different"
        plt.plot(np.arange(len(freq_input)), list(freq_output.values()), linewidth='1', label='{}'.format(n[i]))

    plt.plot(np.arange(len(freq_input)), list(freq_input.values()), linewidth='3', color='k', label='MIDIS')
    leg = plt.legend(loc='best', frameon=False, fontsize=18)
    leg.set_title('Network size',prop={'size':18})
    plt.xticks(np.arange(len(sorted_notes)), sorted_notes, fontsize=10, rotation=90)
    plt.yticks([0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7],
           ['0%', '10%', '20%', '30%', '40%', '50%', '60%', '70%'], fontsize=18)
    plt.ylabel('frequency', fontsize=18)
    if title is not None:
        plt.title(title, fontsize=18)
    plt.tight_layout()
    plt.show()
    if fig_path is not None:
        save_path = fig_path+'n_params_spont_output_'+exp_sup_name+'_symbollength_'+str(n_gram)+'.pdf'
        plt.savefig(save_path, format='pdf')
    return


def plot_netsize_vs_freq_spont_music_polyphonic(res_path, exp_sup_name, remove_infrequent_ones=False, fig_path=None, title=None):
    first = True
    perf = []
    n = []
    for i in os.listdir(res_path):
        if i.startswith(exp_sup_name+'_'):
            perf.append(pickle.load(open(os.path.join(res_path,i, 'score_spontaneous.obj'), 'rb')))
            config = configparser.ConfigParser()
            config.read(os.path.join(res_path,i,'config.ini'))
            n.append(config['Parameters']['n_e'])

    if remove_infrequent_ones: 
        # clean the dictionaries such that they are the same (we delete occurences that have frequency below 10e-5)
        for i in range(len(perf)):
            for k, v in list(perf[i]['input_frequencies'].items()):
                if v < 10e-4:
                    del perf[i]['input_frequencies'][k]
                    del perf[i]['output_frequencies'][k]
                    perf[i]['sorted_symbols'].remove(k)
    # sort them
    n_int = [int(x) for x in n]
    indices = np.argsort(n_int)
    perf = [perf[x] for x in indices]
    n = [n[x] for x in indices]

    for i in range(len(perf)):
        if first: 
            freq_input = perf[i]['input_frequencies']
            sorted_notes = perf[i]['sorted_symbols']
            first = False
        freq_output = perf[i]['output_frequencies']
        assert list(freq_output.keys()) == list(freq_input.keys()), "input and output dictionaries are different"
        plt.plot(np.arange(len(freq_input)), list(freq_output.values()), linewidth='1', label='{}'.format(n[i]))

    plt.plot(np.arange(len(freq_input)), list(freq_input.values()), linewidth='3', color='k', label='MIDIS')
    leg = plt.legend(loc='best', frameon=False, fontsize=18)
    leg.set_title('Network size',prop={'size':18})
    plt.xticks(np.arange(len(sorted_notes)), sorted_notes, fontsize=10, rotation=90)
    plt.yticks([0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7],
           ['0%', '10%', '20%', '30%', '40%', '50%', '60%', '70%'], fontsize=18)
    plt.ylabel('frequency', fontsize=18)
    if title is not None:
        plt.title(title, fontsize=18)
    plt.tight_layout()
    plt.show()
    if fig_path is not None:
        save_path = fig_path+'n_params_spont_output_poly'+exp_sup_name+'.pdf'
        plt.savefig(save_path, format='pdf')
    return



def plot_netsize_vs_specific_performance_music(res_path, exp_sup_name, fig_path=None, title=None):

    perf_train = []
    perf_test = []
    n = []
    for i in os.listdir(res_path):
        if i.startswith(exp_sup_name+'_'):
            perf_train.append(pickle.load(open(os.path.join(res_path,i, 'score_train.obj'), 'rb')))
            perf_test.append(pickle.load(open(os.path.join(res_path,i, 'score_test.obj'), 'rb')))
            config = configparser.ConfigParser()
            config.read(os.path.join(res_path,i,'config.ini'))
            n.append(config['Parameters']['n_e'])
    width = 0.2

    n_int = [int(x) for x in n]
    indices = np.argsort(n_int)
    sorted_perf_train = [perf_train[x] for x in indices]
    sorted_perf_test = [perf_test[x] for x in indices]
    xticks = list(sorted_perf_train[0].keys())
    n = [n[x] for x in indices]

    fig=plt.figure(figsize=(10,5))
    if title is not None: 
        plt.suptitle(title, fontsize=15)
    plt.subplot(121)
    plt.title('Training set', fontsize=15)
    for i in range(len(sorted_perf_train)):
        plt.bar(np.arange(len(xticks))+ (i-1)*width, [x for x in sorted_perf_train[i].values()], width=width, alpha=0.7)

    plt.ylabel('correct predictions', fontsize=15)
    plt.ylim([0, 1])

    plt.xticks(np.arange(len(xticks)), xticks, fontsize=15)
    plt.yticks([0, 0.2, 0.4, 0.6, 0.8, 1.0],
            ['0%', '20%', '40%', '60%', '80%', '100%',], fontsize=15)

    plt.subplot(122)
    xticks = list(sorted_perf_test[0].keys())

    plt.title('Test set', fontsize=15)
    for i in range(len(sorted_perf_train)):
        plt.bar(np.arange(len(xticks))+ (i-1)*width, [x for x in sorted_perf_test[i].values()], width=width, alpha=0.7, label='{}'.format(n[i]))

    leg = fig.legend(loc='lower center', fontsize=15, ncol=len(n),bbox_to_anchor=(0.5, -0.03))
    leg.set_title('Network size', prop={'size':15})
    plt.xticks(np.arange(len(xticks)), xticks, fontsize=15)
    plt.yticks([0, 0.2, 0.4, 0.6, 0.8, 1.0],
            ['0%', '20%', '40%', '60%', '80%', '100%',], fontsize=15)
    plt.ylim([0, 1])
    fig.subplots_adjust(bottom=0.2)

    if fig_path is not None:
        save_path = fig_path+'spec_perf_'+exp_sup_name+'.pdf'
        plt.savefig(save_path, format='pdf')

    return

def plot_specific_performance(spec_perf, path=None, title=None): 
    'spec freq: dictionary with specific frequency per symbol'
    fig = plt.figure(1, figsize=(15, 6))

    xticks = list(spec_perf.keys())
    plt.bar(np.arange(len(xticks)), [x for x in spec_perf.values()],alpha=0.7)

    plt.xticks(np.arange(len(xticks)), xticks, fontsize=15)
    plt.yticks([0, 0.2, 0.4, 0.6, 0.8, 1.0],
           ['0%', '20%', '40%', '60%', '80%', '100%',], fontsize=15)
    plt.ylim([0, 1])
    plt.ylabel('correct predictions', fontsize=15)
    if title is not None: 
        plt.title(str(title), fontsize=15)

    if path is not None:
        save_path = path+'specific_performance.pdf'
        print(save_path)
        plt.savefig(save_path, format='pdf')
    plt.show()



def plot_frequencies(score_dict, ngram=1, path=None, title=None):
    ''' compares frequencies of symbols occuring in corpus vs. in output, 
        parameter ngram stands for length of symbol sequence compared'''

    freq_output = score_dict['output_frequencies_{}'.format(ngram)]
    freq_input = score_dict['input_frequencies_{}'.format(ngram)]
    sorted_notes = score_dict['sorted_symbols_{}'.format(ngram)]

    plt.plot(np.arange(len(freq_input)), list(freq_output.values()), linewidth='1', label='output')
    plt.plot(np.arange(len(freq_input)), list(freq_input.values()), linewidth='3', color='k', label='input')

    leg = plt.legend(loc='best', frameon=False, fontsize=18)
    plt.xticks(np.arange(len(sorted_notes)), sorted_notes, fontsize=10, rotation=90)
    plt.yticks([0, 0.1, 0.2, 0.3, 0.4],
           ['0%', '10%', '20%', '30%', '40%'], fontsize=18)
    #plt.ylim([0, 0.4])
    plt.ylabel('frequency', fontsize=18)
    plt.tight_layout()
    if title is not None: 
        plt.title(str(title), fontsize=18)

    if path is not None:
        save_path = path+'frequencies_symbollength{}.pdf'.format(ngram)
        print(save_path)
        plt.savefig(save_path, format='pdf')
    plt.show()   

    return

def plot_frequencies_poly(score_dict,path=None, title=None, show=False):
    ''' compares frequencies of symbols occuring in corpus vs. in output'''

    freq_output = score_dict['output_frequencies']
    freq_input = score_dict['input_frequencies']
    sorted_notes = score_dict['sorted_symbols']

    plt.plot(np.arange(len(freq_input)), list(freq_output.values()), linewidth='1', label='output')
    plt.plot(np.arange(len(freq_input)), list(freq_input.values()), linewidth='3', color='k', label='input')

    leg = plt.legend(loc='best', frameon=False, fontsize=18)
    plt.xticks(np.arange(len(sorted_notes)), sorted_notes, fontsize=10, rotation=90)
    plt.yticks([0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7],
           ['0%', '10%', '20%', '30%', '40%', '50%', '60%', '70%'], fontsize=18)
    #plt.ylim([0, 0.4])
    plt.ylabel('frequency', fontsize=18)
    if title is not None: 
        plt.title(str(title), fontsize=18)
    plt.tight_layout()

    if path is not None:
        save_path = path+'frequencies_polyphonic.pdf'
        print(save_path)
        plt.savefig(save_path, format='pdf')

    if show: 
        plt.show()   

    return


def plot_frequencies_chords_poly(score_dict,path=None, title=None):
    ''' compares frequencies of chords occuring in corpus vs. in output'''

    freq_output = score_dict['output_frequencies_chords']
    freq_input = score_dict['input_frequencies_chords']
    sorted_notes = score_dict['sorted_chords']

    plt.plot(np.arange(len(freq_input)), list(freq_output.values()), linewidth='1', label='output')
    plt.plot(np.arange(len(freq_input)), list(freq_input.values()), linewidth='3', color='k', label='input')

    leg = plt.legend(loc='best', frameon=False, fontsize=18)
    plt.xticks(np.arange(len(sorted_notes)), sorted_notes, fontsize=10, rotation=90)
    plt.yticks([0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7],
           ['0%', '10%', '20%', '30%', '40%', '50%', '60%', '70%'], fontsize=18)
    #plt.ylim([0, 0.4])
    plt.ylabel('frequency', fontsize=18)
    plt.tight_layout()
    if title is not None: 
        plt.title(str(title))

    if path is not None:
        save_path = path+'frequencies_chords_polyphonic.pdf'
        print(save_path)
        plt.savefig(save_path, format='pdf')
    plt.show()   

    return


def plot_(tag, run_id):
    sm = StorageManager(tag, tag + '_{}'.format(run_id))
    #sm = StorageManager('weight_development', 'weight_development_3')  # 'CountingTest_10_7285' default_hist_10_9896 OneLayerSmallFSNW_10_4831
    # sm = StorageManager('just biggest syn test', 'just biggest syn test_40')
    y_values0 = []
    y_values1 = []
    y_values2 = []
    y_values3 = []
    x_values = []

    for i in range(100):
        synapses = sm.load_obj('syn{}'.format(i))

        y = np.average(np.sum(synapses.transpose() > (np.max(synapses, axis=1) * (1 / 2)), axis=0))
        y_values0.append(y)

        y = np.average(np.sum(synapses.transpose() > (np.max(synapses, axis=1) * (1 / 3)), axis=0))
        y_values1.append(y)

        y = np.average(np.sum(synapses.transpose() > (np.max(synapses, axis=1) * (1 / 4)), axis=0))
        y_values2.append(y)

        y = np.average(np.sum(synapses.transpose() > (np.max(synapses, axis=1) * (1 / 5)), axis=0))
        y_values3.append(y)

        x_values.append(i * 1000)

    plt.plot(x_values, y_values3, label='syn>max_syn*(1/5)')
    plt.plot(x_values, y_values2, label='syn>max_syn*(1/4)')
    plt.plot(x_values, y_values1, label='syn>max_syn*(1/3)')
    plt.plot(x_values, y_values0, label='syn>max_syn*(1/2)')

    plt.legend(loc='best', frameon=False, fontsize=20)
    plt.xlabel(r'plasticity iterations', fontsize=20)
    plt.ylabel(r'synapses', fontsize=20)

    plt.show()



def sdgsdfkkd():
    lookup_dict = {'n_wrong_sentences': 'wrong', 'IP_Test_diffuse': 'diffuse', 'n_new': 'new', 'n_wrong': 'wrong',
                   'folder_number': 'id', 'IP_Test_New': 'IP', 'IP_Test_New_and_diffuse': 'IP+diffuse'}

    def lookup(key):
        if key in lookup_dict:
            return lookup_dict[key]
        else:
            return key

    # Tags = ['SORN_One_layer', 'SORN_Two_layer', 'SORN_Three_layer']
    # Tags = ['SORN_One_layer', 'Adj_SORN_Two_layer', 'Adj_SORN_Three_layer']
    Tags = ['inh syn', 'random plasticity FDT train', 'random plasticity FDT train 2']
    # Tags = ['inh syn', 'just biggest syn']
    params = ['N_e', 'n_output', 'n_wrong_sentences']  # , 'n_new' #, 'n_wrong_sentences' 'n_wrong_words'

    print(params[1:])

    for tag in Tags:
        smg1 = StorageManagerGroup(tag)
        smg1.sort_by(params[0])
        axis = smg1.get_multi_param_list(params)  # axis[0]=N_e vec, axis[1]=n_output vec, ...
        print(axis)

        # convert from number to percentage
        axis[2] = 100 / axis[1] * axis[2]
        # axis[3] = 100 / axis[1] * axis[3]

        for a, p in zip(axis[2:], params[2:]):  # remove first list (N_e)

            x, avg = smg1.remove_duplicates_get_eval(axis[0], a, 'np.average(a)')
            x, std = smg1.remove_duplicates_get_eval(axis[0], a, 'np.std(a)')

            print(x, avg)

            plt.plot(x, avg, label=lookup(p) + ' ' + lookup(tag))
            plt.fill_between(x, avg - std, avg + std, alpha=0.2)

    # plt.axhline(1/540)

    fig_lettersize = 20
    plt.legend(loc='best', frameon=False, fontsize=fig_lettersize)
    plt.xlabel(lookup(params[0]), fontsize=fig_lettersize)
    plt.ylabel(r'sentences (%)', fontsize=fig_lettersize)

    if True:
        format = 'png'
        plot_sm = StorageManager('Plots', '_'.join(Tags) + '___' + '_'.join(params))
        plot_sm.save_param('Tags', str(Tags))
        plot_sm.save_param('params', str(params))
        plot_sm.save_param('lookup', str(lookup))
        plot_sm.save_param('fig_lettersize', fig_lettersize)
        plt.savefig(plot_sm.absolute_path + 'plot.' + format, format=format)

    plt.show()



def rgkdfjhgd(tag, run_id):
    sm = StorageManager(tag, tag + '_{}'.format(run_id))
    #sm = StorageManager('CHILDES_test', 'CHILDES_test_4')  # 'CountingTest_10_7285' default_hist_10_9896 OneLayerSmallFSNW_10_4831

    spec_perf = sm.load_obj('spec_perf')

    plt.bar(np.arange(len(spec_perf.keys())), spec_perf.values())
    plt.show()



def plot_average(tag, run_id):
    sm = StorageManager(tag, tag + '_{}'.format(run_id))
    #sm = StorageManager('osc_test_no_stdp', 'osc_test_no_stdp_0')
    avg = sm.load_obj('avg')
    avg = avg.reshape(-1, 10).mean(axis=-1)
    plt.plot(np.arange(len(avg)), avg, label='...')

    plt.legend(loc='best', frameon=False, fontsize=20)
    plt.xlabel('plasticity steps', fontsize=20)
    plt.ylabel('average excitatory neuron acticity', fontsize=20)
    plt.show()

#plot_average('reduced_oscillation_tests_default_200_syn_N_e_1000_TI_1000_and_100_and_IP_20perc',3)

def get_lognormal_overlay_plot():
    all = []
    # means=
    for i in range(10):
        logn = np.random.lognormal(mean=1 + (10 - i) / 5, size=1000)
        plt.hist(logn, range=(0, 100), bins=100, alpha=0.5)
        all.append(logn)

    plt.xlim(0, 100)
    plt.show()

    plt.hist(np.array(all).flatten(), range=(0, 100), bins=100)
    plt.xlim(0, 100)
    plt.show()


def get_ISI_plot():
    sm = StorageManager('distribution', 'distribution_0')
    avg = sm.load_obj('avg')
    x = sm.load_obj('x')

    isis = []
    for i in range(100):
        isis.append(SpikeTrain_ISI(x[:, i]))

    plt.hist(isis, bins=50, range=(0, 100))  # , stacked=True

    plt.xlim(0, 100)
    plt.show()





def ssdfasd(tag):
    temporal_avg=1000

    smg = StorageManagerGroup(tag)
    cut_off_freq = 0.1

    # def autocorr(x):
    #    result = np.correlate(x, x, mode='full')
    #    return result[result.size/2:]

    def autocorr(x, t=1):
        return np.corrcoef(np.array([x[:-t], x[t:]]))[0][1]

    for i, sm in enumerate(smg.StorageManagerList):
        c = np.random.rand(3)
        avg = sm.load_obj('avg')  # [0:10000]
        avg2 = avg.reshape(-1, temporal_avg).mean(axis=-1)
        Y = np.fft.fft(avg)
        freq = np.fft.fftfreq(len(avg), 1)

        print(autocorr(avg), autocorr(avg2))
        #print(np.var(np.abs(Y[freq > cut_off_freq])))

        #plt.plot(freq[freq > cut_off_freq], np.abs(Y[freq > cut_off_freq]), alpha=0.5, color=c)
        plt.plot(avg2)

    plt.show()

#temporal average + auto corellation

#ssdfasd('osc_test_no_stdp_big')
#ssdfasd('test')
#ssdfasd('reduced_oscillation_tests_default_100_syn_N_e_1000')


def polygon_under_graph(xlist, ylist):
    return [(xlist[0], 0.)] + list(zip(xlist, ylist)) + [(xlist[-1], 0.)]

#tags = ['reduced_oscillation_tests_default_50_syn', 'reduced_oscillation_tests_10perc_50_syn', 'reduced_oscillation_tests_20perc_50_syn', 'reduced_oscillation_tests_50perc_50_syn', 'reduced_oscillation_tests_80perc_50_syn']
#n=['default', '10%', '20%', '50%', '80%']
#tags = ['reduced_oscillation_tests_def_200_syn', 'reduced_oscillation_tests_10perc_200_syn', 'reduced_oscillation_tests_20perc_200_syn', 'reduced_oscillation_tests_50perc_200_syn']

#tags = ['reduced_oscillation_tests_diffIP_def_50_syn', 'reduced_oscillation_tests_diffIP_10perc_50_syn', 'reduced_oscillation_tests_diffIP_20perc_50_syn', 'reduced_oscillation_tests_diffIP_50perc_50_syn']
#tags = ['reduced_oscillation_tests_diffIP_def_200_syn', 'reduced_oscillation_tests_diffIP_10perc_200_syn', 'reduced_oscillation_tests_diffIP_20perc_200_syn', 'reduced_oscillation_tests_diffIP_50perc_200_syn']

#tags = ['osc_test_no_stdp', 'osc_test_stdp']
#tags = ['osc_test_no_stdp_big', 'osc_test_stdp_big']

def show_3D_oscillation_plot(tags, temporal_avg=10):


    for tn, tag in enumerate(tags):

        cc = 'rgbyrgbyrgbyrgbyrgbyrgbyrgbyrgbyrgbyrgby'

        smg = StorageManagerGroup(tag)
        cut_off_freq = 0.1
        c = np.random.rand(3)

        z = []
        colors = []
        verts = []
        fig = plt.figure(num=tag)
        ax = fig.gca(projection='3d')

        for i, sm in enumerate(smg.StorageManagerList):
            if sm.has_obj('avg'):
                z.append(i)
                colors.append(mcolors.to_rgba(cc[i], alpha=0.9))  # np.random.rand(3)
                avg = sm.load_obj('avg')  # [0:10000]
                avg = avg.reshape(-1, temporal_avg).mean(axis=-1)
                # Y = np.fft.fft(avg)
                # freq = np.fft.fftfreq(len(avg), 1)
                # plt.plot(freq[freq > cut_off_freq], np.abs(Y[freq > cut_off_freq]), alpha=0.5, color=c)
                # plt.plot(np.arange(len(avg)), avg, label='50 syn default')
                verts.append(polygon_under_graph(np.linspace(0., len(avg)*temporal_avg, len(avg)), avg))

        # plt.plot([], label=n[tn], color=c)

        poly = PolyCollection(verts, facecolors=colors)
        ax.add_collection3d(poly, zs=z, zdir='y')

        ax.set_xlim(0, len(avg)*temporal_avg)
        ax.set_ylim(-1, i)
        ax.set_zlim(0, 0.5)

        plt.show()

    #plt.legend(loc='best', frameon=False, fontsize=20)
    #plt.xlabel('freq', fontsize=20)
    #plt.ylabel('', fontsize=20)
    #plt.show()




def show_2D_oscillation_plot(tag, run_id, temporal_avg=10):
    sm = StorageManager(tag, tag+'_{}'.format(run_id))
    avg = sm.load_obj('avg')
    avg = avg.reshape(-1, temporal_avg).mean(axis=-1)
    plt.plot(np.arange(len(avg))*temporal_avg, avg, label='50 syn default')

    plt.legend(loc='best', frameon=False, fontsize=20)
    plt.xlabel('plasticity steps', fontsize=20)
    plt.ylabel('average excitatory neuron acticity', fontsize=20)
    plt.show()


def aagdsfgsdf(tag, run_id):
    sm = StorageManager(tag, tag+'_{}'.format(run_id))
    avg = sm.load_obj('avg')

    plt.plot(avg, label='net average')

    # plt.plot(x[:,1], label='neuron act')

    # x2 = x[:,1].reshape(-1, 10).mean(axis=-1)
    # plt.plot(np.arange(len(x2))*10, x2, label='10 step average')

    x2 = x[:, 4].reshape(-1, 100).mean(axis=-1)
    plt.plot(np.arange(len(x2)) * 100, x2, label='1 neuron 100 step average')

    # plt.plot(np.arange(len(x)), x[:,1], label='50 syn default')

    plt.legend(loc='best', frameon=False, fontsize=20)
    plt.xlabel('time', fontsize=20)
    plt.ylabel('', fontsize=20)

    plt.show()


def sdgsdfgdf():
    sm = StorageManager('syn_avg_act', 'syn_avg_act_2')
    avg = sm.load_obj('avg')
    plt.plot(np.arange(len(avg)), avg)
    plt.show()

    scale = 10
    for i in range(3):
        avg = avg.reshape(-1, scale).mean(axis=-1)
        plt.plot(np.arange(len(avg)) * np.power(scale, (i + 1)), avg)
    plt.show()



def bhbhjjb():
    cut_off_freq = 0.1

    sm = StorageManager('syn_avg_act_no_bug', 'syn_avg_act_no_bug_3')
    avg = sm.load_obj('avg')
    print(len(avg))
    Y = np.fft.fft(avg)
    freq = np.fft.fftfreq(len(avg), 1)
    plt.plot(freq[freq > cut_off_freq], np.abs(Y[freq > cut_off_freq]), label='150 synapses', alpha=0.5)
    # plt.plot(avg, label='150 synapses', alpha=0.5)

    sm = StorageManager('syn_avg_act_no_bug', 'syn_avg_act_no_bug_2')
    avg = sm.load_obj('avg')
    print(len(avg))
    Y = np.fft.fft(avg)
    freq = np.fft.fftfreq(len(avg), 1)
    plt.plot(freq[freq > cut_off_freq], np.abs(Y[freq > cut_off_freq]), label='100 synapses', alpha=0.5)
    # plt.plot(avg, label='100 synapses', alpha=0.5)

    sm = StorageManager('syn_avg_act_no_bug', 'syn_avg_act_no_bug_1')
    avg = sm.load_obj('avg')
    print(len(avg))
    Y = np.fft.fft(avg)
    freq = np.fft.fftfreq(len(avg), 1)
    plt.plot(freq[freq > cut_off_freq], np.abs(Y[freq > cut_off_freq]), label='50 synapses', alpha=0.5)
    # plt.plot(avg, label='50 synapses', alpha=0.5)

    sm = StorageManager('syn_avg_act_no_bug', 'syn_avg_act_no_bug_0')
    avg = sm.load_obj('avg')
    print(len(avg))
    Y = np.fft.fft(avg)
    freq = np.fft.fftfreq(len(avg), 1)
    plt.plot(freq[freq > cut_off_freq], np.abs(Y[freq > cut_off_freq]), label='10 synapses', alpha=0.5)
    # plt.plot(avg, label='10 synapses', alpha=0.5)

    fig_lettersize = 20
    plt.legend(loc='best', frameon=False, fontsize=fig_lettersize)
    plt.xlabel('frequency', fontsize=fig_lettersize)
    plt.ylabel('fourier transform y', fontsize=fig_lettersize)

    plt.show()


def create_beat_matrix_from_results(res_path, offtoken=True, ontoken=False):
    #TODO: need to check that indexing is safe if start/stop tokens occur at first/last time step of output
    perf = pickle.load(open(os.path.join(res_path, 'score_spontaneous.obj'), 'rb'))
    beat_matrix = perf['output_beatmatrix']

    if offtoken and ontoken:
        # last row stands for start token, row before for stop token
        indices_start = np.nonzero(beat_matrix[:,-1])[0]
        indices_stop = np.nonzero(beat_matrix[:,-2])[0]

    elif offtoken and not ontoken: # we only have offtokens, but we start at first generated time step
        indices_stop = np.nonzero(beat_matrix[:,-1])[0]
        indices_start = indices_stop+1

    elif ontoken and not offtoken:
        indices_start = np.nonzero(beat_matrix[:,-1])[0]
        indices_stop = indices_start-1

    indices_start = indices_start[:-1] # remove last start token, it does not have a corresponding stop token
    indices_stop = indices_stop[1:] # remove first stop token, it does not have a corresponding start token

    tracks = []
    if len(indices_start) >0 and len(indices_stop)>0:
        assert [indices_stop[i] > indices_start[i] for i in range(len(indices_start))]

        for i in range(len(indices_start)):
            tracks.append(beat_matrix[indices_start[i]:indices_stop[i]+1,:14])
        
    else:# return whole matrix, we just have one track, it never predicted start/stop
        tracks.append(beat_matrix[:,:14])

    return np.array(tracks)


def get_percussion_track_entropy(track, len_alphabet=14, include_silence=False):
    distribution = np.zeros(len_alphabet)
    indices_non_zero = track.nonzero()
    for i in range(len_alphabet):
        distribution[i] = sum(indices_non_zero[1] == i)
    if include_silence:
    # add the silent time steps (when no note is played)
        non_zero_indices = set(indices_non_zero[0])
        all_indices = set(range(len(track)))
        silent_indices = all_indices - non_zero_indices
        distribution = np.append(distribution, len(silent_indices))

    return entropy(distribution)

def compute_per_track_entropy(tracks):
        '''
        return a list of per track entropies
        expects a beat matrix of shape (n_tracks, len_alphabet)
        '''
        entropy_generated_tracks = []

        if len(tracks)==1:
            entropy_generated_tracks = self.get_percussion_track_entropy(tracks, tracks.shape[1], include_silence = include_silence)

        else:
            for i in range(len(tracks)):
                entropy_generated_tracks.append(get_percussion_track_entropy(tracks[i], tracks[i].shape[1]))

        return entropy_generated_tracks

def compute_per_track_length(tracks):
    '''
    returns a list of track lengths
    expects a beat matrix of shape (n_tracks, len_alphabet)
    '''
    return [len(i) for i in tracks]






from PymoNNto.Exploration.UI_Base import *
from PymoNNto.Exploration.StorageManager.StorageManager import *
from PymoNNto.Exploration.Evolution.EvolutionPlots import *



class UI_Single_Evolution_Monitor(UI_Base):

    def __init__(self, evolution):
        super().__init__(None, label='Evolution Monitor', create_sidebar=False)

        self.evolution = evolution

        self.main_tab = self.Next_Tab('Evolution performance', stretch=0.0)

        self.reduced_layout = False

        self.Next_H_Block(stretch=10)

        add_evolution_plot_items(self, self.main_tab)

        self.Next_H_Block(stretch=0.0)
        self.pause_cont_btn = self.Add_element(QPushButton('Pause'), stretch=0)
        self.pause_cont_btn.clicked.connect(self.on_pause_cont_click)

        #self.Add_element(QLabel('...'), stretch=0)

        self.last_generation = -1

        self.timer = QtCore.QTimer(self)
        self.timer.timeout.connect(self.on_timer)
        self.timer.start(40)

    def on_pause_cont_click(self):
        if self.timer.isActive():
            self.timer.stop()
            self.pause_cont_btn.setText('Start')
            print('stopped (only the main distribution timer is stopped but the remaining threads are still running to complete their current jobs)')
        else:
            self.timer.start(40)
            self.pause_cont_btn.setText('Pause')
            print('started')

    def on_timer(self):
        self.evolution.iteration()

    def update(self):
        if self.last_generation != self.evolution.Breed_And_Select.generation:
            update_evolution_plot(self, self.main_tab, self.evolution.name, self.evolution.gene_keys)
            self.last_generation = self.evolution.Breed_And_Select.generation



from sympy import symbols
import sympy.physics.units as units
from sympy.physics.units import *

def eq_split(eq):
    str=eq.replace(' ', '')
    parts = []
    str_buf = ''
    for s in str:
        if s in ['*', '/', '+', '-', '%', ':', ';', '=', '!', '(', ')', '[', ']']:
            parts.append(str_buf)
            parts.append(s)
            str_buf = ''
        else:
            str_buf += s

    parts.append(str_buf)
    return parts

def remove_units(eq_parts, i):

    while i < len(eq_parts):

        if eq_parts[i] == 'ms':

            t = float(convert_to(eval(eq_parts[i - 2] + eq_parts[i - 1] + eq_parts[i]), seconds) / second)

            eq_parts[i - 2] = '{}'.format(t)

            # eq_parts[i-2] = '1000/'+eq_parts[i-2]
            eq_parts.pop(i)
            eq_parts.pop(i - 1)
            i -= 2

        if eq_parts[i] in units.__dict__ or eq_parts[i] in myUnits:  # remove units
            eq_parts.pop(i)
            eq_parts.pop(i - 1)
            i -= 2
        i += 1

    return eq_parts

myUnits=['mV']


from PymoNNto.Exploration.Network_UI.TabBase import *
from PymoNNto.Exploration.Visualization.Visualization_Helper import *

class criticality_tab(TabBase):

    def __init__(self, title='Criticality', param='output', mask_param='Input_Mask', timesteps=500, mask_color_add=(-100, -100, -100)):
        super().__init__(title)
        self.param = param
        self.timesteps=timesteps
        self.mask_param = mask_param
        if self.mask_param is not None:
            self.compiled_param = compile('n.'+self.mask_param, '<string>', 'eval')
            self.inverted_compiled_param = compile('np.invert(n.' + self.mask_param+')', '<string>', 'eval')
            self.mask_color_add = mask_color_add

    def add_recorder_variables(self, neuron_group, Network_UI):
        if hasattr(neuron_group, self.param):
            Network_UI.add_recording_variable(neuron_group, 'n.' + self.param, timesteps=self.timesteps)
        if hasattr(neuron_group, self.param):
            self.sum_tag='np.sum(n.'+self.param+')'
            Network_UI.add_recording_variable(neuron_group, self.sum_tag, timesteps=1000)

    def initialize(self, Network_UI):
        self.criticality_tab = Network_UI.Next_Tab(self.title)

        self.avalance_size_curve, self.avalance_size_plot = Network_UI.Add_plot_curve('net avalance size plot', True, False, legend=False, x_label='Size', y_label='Probability')
        self.avalance_duration_curve, self.avalance_duration_plot = Network_UI.Add_plot_curve('net avalance duration plot', True, False, legend=False, x_label='Duration', y_label='Probability')


        #move to update...
        self.avalance_size_plot.setXRange(min=0, max=100, padding=0)
        self.avalance_size_plot.setXRange(min=0, max=100, padding=0)

        self.log = True
        self.avalance_size_plot.setLogMode(self.log, self.log)
        self.avalance_duration_plot.setLogMode(self.log, self.log)


        _, self.group_branche_estimate_plt = Network_UI.Add_plot_curve('Group Wilting Branching Estimate', True, False, number_of_curves=2, legend=False, x_label='delta t', y_label='Autocorrelation r delta t')

        self.group_scatter=pg.ScatterPlotItem()
        self.group_estimate=pg.PlotCurveItem(pen=(255,0,0))
        self.group_m_text = pg.TextItem(text='...', anchor=(0, 0))
        self.group_m_text.setPos(75, 0.5)
        self.group_test_text = pg.TextItem(text='...', anchor=(0, 0))
        self.group_test_text.setPos(75, 0.4)

        self.group_branche_estimate_plt.addItem(self.group_scatter)
        self.group_branche_estimate_plt.addItem(self.group_estimate)
        self.group_branche_estimate_plt.addItem(self.group_m_text)
        self.group_branche_estimate_plt.addItem(self.group_test_text)

        _, self.net_branche_estimate_plt = Network_UI.Add_plot_curve('Network Wilting Branching Estimate', True, False, number_of_curves=2, legend=False, x_label='delta t', y_label='Autocorrelation r delta t')

        self.net_scatter=pg.ScatterPlotItem()
        self.net_estimate=pg.PlotCurveItem(pen=(255,0,0))
        self.net_m_text = pg.TextItem(text='...', anchor=(0, 0))
        self.net_m_text.setPos(75, 0.5)
        self.net_test_text = pg.TextItem(text='...', anchor=(0, 0))
        self.net_test_text.setPos(75, 0.4)

        self.net_branche_estimate_plt.addItem(self.net_scatter)
        self.net_branche_estimate_plt.addItem(self.net_estimate)
        self.net_branche_estimate_plt.addItem(self.net_m_text)
        self.net_branche_estimate_plt.addItem(self.net_test_text)

        self.WP_test_execute = False
        def on_click(event):
            self.WP_test_execute = True
            #self.update_branching(Network_UI.selected_neuron_group())
            #self.log = not self.log
            #self.avalance_size_plot.setLogMode(self.log, self.log)
            #self.avalance_duration_plot.setLogMode(self.log, self.log)

        btn = QPushButton('WP Test')
        btn.mousePressEvent = on_click
        Network_UI.Add_element(btn)

        Network_UI.Next_H_Block(stretch=0.1)

        self.min_slider = QSlider(1)  # QtCore.Horizontal
        self.min_slider.setMinimum(1)
        self.min_slider.setMaximum(100)
        self.min_slider.setSliderPosition(1)
        self.min_slider.mouseReleaseEvent = Network_UI.static_update_func
        self.min_slider.setToolTip('slide to cut away smallest slopes')
        Network_UI.Add_element(self.min_slider)  # , stretch=0.1

        Network_UI.Next_H_Block(stretch=0.1)

        self.split_slider = QSlider(1)  # QtCore.Horizontal
        self.split_slider.setMinimum(1)
        self.split_slider.setMaximum(10)
        self.split_slider.setSliderPosition(1)
        self.split_slider.mouseReleaseEvent = Network_UI.static_update_func
        self.split_slider.setToolTip('split to calculate average block')
        Network_UI.Add_element(self.split_slider)  # , stretch=0.1

        self.WP_init = False
        self.WPT_init = False

    def button_update(self, Network_UI, group):
        if self.WP_test_execute:
            self.WP_test_execute = False

            #if self.WPT_init:
            import PymoNNto.Exploration.Analysis.WP_testing as WPT
            import mrestimator as mre
            #self.WPT_init = True

            group_A_t = group[self.sum_tag, 0, 'np'].copy()  # [-1000:]
            net_A_t = np.sum(Network_UI.network[self.sum_tag], axis=0)  # np.swapaxes(, 0, 1)#

            rk, ft, (counts, bins) = WPT.branching_ratio(group_A_t, 1)
            expoffset_fit = mre.fit(rk, fitfunc=mre.f_exponential_offset)
            test1 = WPT.h_offset(ft, expoffset_fit)
            test2 = WPT.h_tau(ft, expoffset_fit)
            test3 = WPT.h_lin(rk, ft)
            print(str(test1)+' '+str(test2)+' '+str(test3))

            rk, ft, (counts, bins) = WPT.branching_ratio(net_A_t, 1)
            expoffset_fit = mre.fit(rk, fitfunc=mre.f_exponential_offset)
            test1 = WPT.h_offset(ft, expoffset_fit)
            test2 = WPT.h_tau(ft, expoffset_fit)
            test3 = WPT.h_lin(rk, ft)
            print(str(test1)+' '+str(test2)+' '+str(test3))



    def update_branching(self, Network_UI, group):

        #if not self.WP_init:
        import PymoNNto.Exploration.Analysis.WiltingPriesemann as WP
        #self.WP_init = True

        group_A_t = group[self.sum_tag, 0, 'np'].copy()[-1000:]

        net_A_t = np.sum(Network_UI.network[self.sum_tag], axis=0)#np.swapaxes(, 0, 1)#

        #print(len(group_A_t), len(net_A_t))

        k_min = self.min_slider.sliderPosition()
        k_max = min(150, len(group_A_t)-10)

        fractions = self.split_slider.sliderPosition()

        if k_max > 0:

            group_mr_A = WP.MR_estimation(group_A_t, k_min, k_max, fractions=fractions)
            self.group_scatter.setData(group_mr_A['k'], group_mr_A['r_k'])
            self.group_estimate.setData(group_mr_A['k'], group_mr_A['fitfunc'](group_mr_A['k'], *group_mr_A['p_opt']))
            self.group_m_text.setText(str(group_mr_A['branching_ratio']))


            net_mr_A = WP.MR_estimation(net_A_t, k_min, k_max, fractions=fractions)
            self.net_scatter.setData(net_mr_A['k'], net_mr_A['r_k'])
            self.net_estimate.setData(net_mr_A['k'], net_mr_A['fitfunc'](net_mr_A['k'], *net_mr_A['p_opt']))
            self.net_m_text.setText(str(net_mr_A['branching_ratio']))



    def update_avalanche_distributions(self, Network_UI, group):

        activity = group[self.sum_tag, 0, 'np']

        theta = 'half_mean'

        if theta == 'half_mean':
            theta = activity.mean() / 2.

        thresholded_activity = activity - theta
        length_act = len(activity)
        duration_list = []
        size_list = []

        duration = 0
        size = 0
        for i in range(length_act):

            # add one time step to the current avalanche
            if thresholded_activity[i] > 0:
                duration += 1
                size += int(thresholded_activity[i])

            # finish current avalanche and prepare for the next one
            elif size != 0:
                duration_list.append(duration)
                size_list.append(size)
                duration = 0
                size = 0

        T_data = np.asarray(duration_list)
        S_data = np.asarray(size_list)

        # 4. duration distribution
        T_x, T_inverse = np.unique(T_data, return_inverse=True)
        T_y_freq = np.bincount(T_inverse)
        T_y = T_y_freq / float(T_y_freq.sum())  # normalization

        if self.log:
            T_x = np.log(T_x)
            T_y = np.log(T_y)

        self.avalance_duration_curve.setData(T_x, T_y)



        # 5. size distribution
        #fig_b = plt.subplot(122)
        S_x, S_inverse = np.unique(S_data, return_inverse=True)
        S_y_freq = np.bincount(S_inverse)
        S_y = S_y_freq / float(S_y_freq.sum())  # normalization

        if self.log:
            S_x = np.log(S_x)
            S_y = np.log(S_y)

        #print(S_x, S_y)
        self.avalance_size_curve.setData(S_x, S_y)


    def update(self, Network_UI):
        if self.criticality_tab.isVisible():

            group = Network_UI.selected_neuron_group()
            n = group#for eval comand

            if self.mask_param is not None and hasattr(group, self.mask_param):
                input_mask = eval(self.compiled_param)
                not_input_mask = eval(self.inverted_compiled_param)
                mca = self.mask_color_add
            else:
                input_mask = False
                not_input_mask = True
                mca = (0, 0, 0)

            net_color_input = np.clip([group.color[0] + mca[0], group.color[1] + mca[1], group.color[2] + mca[2], 255], 0, 255)

            if hasattr(group, self.param):
                self.update_avalanche_distributions(Network_UI, group)
                self.update_branching(Network_UI, group)
                self.button_update(Network_UI, group)



        #T_fit = pl.Fit(T_data, xmin=6, xmax=60, discrete=True)
        #T_alpha = T_fit.alpha
        #T_sigma = T_fit.sigma
        #T_xmin = T_fit.xmin

        #plt.plot(T_x, T_y, '.', markersize=2)
        #pl.plot_pdf(T_data)
        #T_fit.power_law.plot_pdf(label=r'$\alpha = %.2f$' % T_alpha)

        #fig_lettersize = 12
        #plt.title('Neuronal Avalanches - duration distribution')
        #plt.legend(loc='best')
        #plt.xlabel(r'$T$', fontsize=fig_lettersize)
        #plt.ylabel(r'$f(T)$', fontsize=fig_lettersize)
        #plt.xscale('log')
        #plt.yscale('log')


        #S_fit = pl.Fit(S_data, xmin=10, xmax=1500, discrete=True)
        #S_alpha = S_fit.alpha
        #S_sigma = S_fit.sigma
        #S_xmin = S_fit.xmin

        #plt.plot(S_x, S_y, '.', markersize=2)
        #pl.plot_pdf(S_data)
        #S_fit.power_law.plot_pdf(label=r'$\tau = %.2f$' % S_alpha)

        #fig_lettersize = 12
        #plt.title('Neuronal Avalanches - size distribution')
        #plt.legend(loc='best')
        #plt.xlabel(r'$S$', fontsize=fig_lettersize)
        #plt.ylabel(r'$f(S)$', fontsize=fig_lettersize)
        #plt.xscale('log')
        #plt.yscale('log')


from PymoNNto.Exploration.UI_Base import *
from PymoNNto.Exploration.StorageManager.StorageManager import *
from PymoNNto.Exploration.Evolution.SSH_Functions import *
from PymoNNto.Exploration.Evolution.EvolutionPlots import *
from multiprocessing import Process, Queue, Pipe
import shutil

class QHLine(QFrame):
    def __init__(self):
        super(QHLine, self).__init__()
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)

class QVLine(QFrame):
    def __init__(self):
        super(QVLine, self).__init__()
        self.setFrameShape(QFrame.VLine)
        self.setFrameShadow(QFrame.Sunken)

def evolution_thread_worker(name, python_cmd, conn):
    print('local thread started')
    try:
        dir = os.getcwd()
        os.chdir(get_epc_folder()+'/'+name)

        system_str = sys.platform
        if 'darwin' in system_str:#mac os
            os.system(python_cmd+' execute_evolution.py')#todo: test dfdsgdf
        elif 'win' in system_str:#windows
            os.system('start /wait cmd /c '+python_cmd+' execute_evolution.py')
        else:
            os.system('gnome-terminal -x '+python_cmd+' execute_evolution.py')

        os.chdir(dir)

        conn.send('execution finished')
    except:
        conn.send('thread error')
    conn.send('idle')


class UI_Evolution_Manager(UI_Base):

    def __init__(self):
        super().__init__(None, label='Evolution Monitor', create_sidebar=True)

        self.tabs.currentChanged.connect(self.On_Tab_Changed)

        ssm = SimpleStorageManager(get_epc_folder()+'/')
        servers = eval(ssm.load_param('servers', default="[]"))#['local', 'ssh vieth@poppy.fias.uni-frankfurt.de', 'ssh marius@hey3kmuagjunsk2b.myfritz.net', '+']
        self.listwidget2 = QListWidget()
        self.listwidget2.addItems(['+', 'local']+servers)
        self.listwidget2.currentItemChanged.connect(self.on_server_select)
        self.Add_element(self.listwidget2, stretch=10.0, sidebar=True)

        horizontal_layout = QHBoxLayout()
        self.sidebar_current_vertical_layout.addLayout(horizontal_layout)

        vertical_layout = QVBoxLayout()
        horizontal_layout.addLayout(vertical_layout)
        vertical_layout.addWidget(QLabel('evo name'))
        vertical_layout.addWidget(QLabel('slave file'))
        vertical_layout.addWidget(QLabel('thread number'))
        vertical_layout.addWidget(QLabel('python cmd'))
        vertical_layout.addWidget(QLabel('individual count'))
        vertical_layout.addWidget(QLabel('mutation'))
        vertical_layout.addWidget(QLabel('death rate'))
        vertical_layout.addWidget(QLabel('start geneomes'))
        vertical_layout.addWidget(QLabel('inactive info'))
        vertical_layout.addWidget(QLabel('constriants'))
        vertical_layout.addWidget(QLabel('evo options'))

        self.evo_name_edit = QLineEdit('evo_name')
        self.slave_file_edit = QLineEdit('Exploration/Evolution/test_slave.py')
        self.thread_number_edit = QLineEdit('4')
        self.python_cmd_edit = QLineEdit("python3")
        self.individual_count_edit = QLineEdit('10')
        self.mutation_edit = QLineEdit('0.05')
        self.death_rate_edit = QLineEdit('0.5')
        self.start_genomes_edit = QLineEdit("[{'a':1,'b':2,'c':2,'d':2,'e':3}]")
        self.inactive_genome_info_edit = QLineEdit('{}')
        self.constraints_edit = QLineEdit("['a<b','b<=c']")
        self.evo_options_edit = QLineEdit('{}')


        vertical_layout = QVBoxLayout()
        horizontal_layout.addLayout(vertical_layout)
        vertical_layout.addWidget(self.evo_name_edit)
        vertical_layout.addWidget(self.slave_file_edit)
        vertical_layout.addWidget(self.thread_number_edit)
        vertical_layout.addWidget(self.python_cmd_edit)
        vertical_layout.addWidget(self.individual_count_edit)
        vertical_layout.addWidget(self.mutation_edit)
        vertical_layout.addWidget(self.death_rate_edit)
        vertical_layout.addWidget(self.start_genomes_edit)
        vertical_layout.addWidget(self.inactive_genome_info_edit)
        vertical_layout.addWidget(self.constraints_edit)
        vertical_layout.addWidget(self.evo_options_edit)

        self.add_btn = self.Add_element(QPushButton('Add'), sidebar=True, stretch=1.0)
        self.add_btn.clicked.connect(self.on_add_click)

        for dir in os.listdir(get_epc_folder()):
            if os.path.isdir(get_epc_folder()+'/'+dir):
                self.add_tab(dir)

    def keyPressEvent(self, event):
        if event.key() in [Qt.Key_I]:
            tab = self.tabs.currentWidget()
            print(tab.clicked_id)
            smg = StorageManagerGroup(tab.name, data_folder=get_data_folder() + '/Evolution_Project_Clones/' + tab.name + '/Data')
            smg.sort_by('gen')
            sm = smg.StorageManagerList[tab.clicked_id]
            file = sm.absolute_path + sm.config_file_name
            print(file)
            with open(file) as f:
                lines = f.readlines()

                layout = QVBoxLayout()
                pte = QPlainTextEdit()
                pte.setPlainText('\r\n'.join(lines))
                pte.setReadOnly(True)
                layout.addWidget(pte)
                dlg = QDialog()
                dlg.setWindowTitle("Info")
                dlg.setLayout(layout)
                dlg.resize(600, 400)
                dlg.exec()

    def set_text(self, ssm, edit, text):
        data = ssm.load_param(text, default='', return_string=True)
        if data is not None and data != '':
            edit.setText(data)

    def On_Tab_Changed(self, i):
        if i is not None:
            tab = self.tabs.currentWidget()
            if hasattr(tab, 'ssm'):
                self.update_status(tab, tab.name)
                ssm = self.tabs.currentWidget().ssm
                self.set_text(ssm, self.evo_name_edit, 'name')
                self.set_text(ssm, self.slave_file_edit, 'slave_file')
                self.set_text(ssm, self.inactive_genome_info_edit, 'inactive_genome_info')
                self.set_text(ssm, self.thread_number_edit, 'thread_number')
                self.set_text(ssm, self.individual_count_edit, 'individual_count')
                self.set_text(ssm, self.mutation_edit, 'mutation')
                self.set_text(ssm, self.death_rate_edit, 'death_rate')
                self.set_text(ssm, self.start_genomes_edit, 'start_genomes')
                self.set_text(ssm, self.constraints_edit, 'constraints')
                self.set_text(ssm, self.python_cmd_edit, 'python_cmd')
                self.set_text(ssm, self.evo_options_edit, 'evo_options')


    def update_status(self, tab, name):
        if 'ssh ' in tab.server:
            if name in ssh_get_running(tab.server):
                tab.status_label.setText('<font color="green">running</font>')
                return True
            else:
                tab.status_label.setText('<font color="red">inactive</font>')
                return False

        if 'local' in tab.server:
            if tab.process is not None:
                if tab.parent_conn.poll():
                    msg = tab.parent_conn.recv()
                    tab.process.kill()
                    tab.process = None
                    tab.status_label.setText('<font color="red">inactive</font>')
                    return False
                else:
                    tab.status_label.setText('<font color="green">running</font>')
                    return True
            else:
                tab.status_label.setText('<font color="red">inactive</font>')
                return False

    def add_tab(self, name):

        folder = get_epc_folder()+'/'+name+'/'

        tab = self.Next_Tab(name, stretch=0.0)
        tab.name=name

        tab.status_label = self.Add_element(QLabel('Status'))

        tab.ssm = SimpleStorageManager(folder)
        tab.server = tab.ssm.load_param('server', default=None)

        tab.gene_keys = tab.ssm.load_param('gene_keys', default=None)
        if tab.gene_keys is not None:
            tab.gene_keys = eval(tab.gene_keys)#should return list like ['a', 'b', 'c', 'd', 'e']  todo:save on evo start

        def refresh():
            try:
                print('refresh')
                if tab.server is not None and 'ssh ' in tab.server:
                    user, host, password = split_ssh_user_host_password_string(tab.server)
                    print('get Data')
                    get_Data(name, user, host, password)
                if tab.gene_keys is not None:
                    print('update plot')
                    update_evolution_plot(self, tab, name, tab.gene_keys, data_folder=get_data_folder() + '/Evolution_Project_Clones/' + name + '/Data')
            except:
                print('no plot data found in', name)

            self.update_status(tab, name)

        refresh_btn = self.Add_element(QPushButton('refresh state'))
        refresh_btn.clicked.connect(refresh)

        tab.process = None

        tab.python_cmd = tab.ssm.load_param('python_cmd', default='python3')

        def start_continue():
            if 'ssh ' in tab.server:
                if not self.update_status(tab, name):
                    ssh_execute_evo(tab.server, name, python_cmd=tab.python_cmd)#python3
                else:
                    print('thread already running')

            if 'local' in tab.server:
                if tab.process is None:
                    tab.parent_conn, child_conn = Pipe()
                    tab.process = Process(target=evolution_thread_worker, args=(name, tab.python_cmd, child_conn))#python
                    tab.process.start()
                else:
                    print('thread already running')

            self.update_status(tab, name)

        start_evo_btn = self.Add_element(QPushButton('start/continue'))
        start_evo_btn.clicked.connect(start_continue)

        def stop():
            if 'ssh ' in tab.server:
                ssh_stop_evo(tab.server, name)
                self.update_status(tab, name)
            if 'local' in tab.server:
                if tab.process is not None:
                    print('Please close terminal manually')
                    tab.process.kill()
                    tab.process.terminate()
                    tab.process.close()
                    tab.process=None


        stop_btn = self.Add_element(QPushButton('stop evolution'))
        stop_btn.clicked.connect(stop)


        def remove():
            if 'ssh ' in tab.server:
                ssh_stop_evo(tab.server, name, remove_evo=True)

            if 'local' in tab.server:
                if tab.process is not None:
                    print('Please close terminal manually')
                    tab.process.kill()
                    tab.process.terminate()
                    tab.process.close()
                    tab.process=None

            shutil.rmtree(get_epc_folder() + '/' + name + '/')
            self.tabs.removeTab(self.tabs.currentIndex())

        remove_btn = self.Add_element(QPushButton('remove'))
        remove_btn.clicked.connect(remove)

        def help():
            layout = QVBoxLayout()
            pte = QPlainTextEdit()
            pte.setPlainText('''Every tab corresponds to a folder in:
Data/Evolution_Project_Clones/
            
For remote SSH servers the local folder only contains some config files while the project copy is located in the user folder of the server.

When the evolution is started on a ssh server it creates a "screen" session which is named after the evolution name.

Useful functions are:

"screen -list" to show the screen sessions and their ids,
"screen -r id" to connect to a session where "id" has to be replaced and
"exit" to close a screen session you are connected to.

The project folders also contain the file execute_evolution.py which is used to start the evolution.

All genomes generate a file with the results and the genes when set_score is called in the slave file. Which are transferred to the local folder when the state is refreshed.
''')
            pte.setReadOnly(True)
            layout.addWidget(pte)

            #self.layout.addWidget(message)
            #self.layout.addWidget(self.buttonBox)

            dlg = QDialog()
            dlg.setWindowTitle("Help")
            dlg.setLayout(layout)
            dlg.resize(600, 400)
            dlg.exec()

        help_btn = self.Add_element(QPushButton('help'))
        help_btn.clicked.connect(help)

        self.Add_element(QLabel(tab.server))

        self.Next_H_Block(stretch=10)

        add_evolution_plot_items(self, tab)

        tab.folder = get_epc_folder() + '/' + name + '/'

        #self.update_status(tab, name)
        self.On_Tab_Changed(None)

    def on_server_select(self):
        server_str = self.listwidget2.currentItem().text()
        if server_str == '+':
            text, ok = QInputDialog.getText(self.main_window, 'Add...', 'Add computing device "ssh user@host" or "ssh user@host password"')
            if ok and text != '+':
                self.listwidget2.addItem(text)
                ssm = SimpleStorageManager(get_epc_folder()+'/')
                print(ssm.absolute_path)
                ssm.save_param('servers', eval(ssm.load_param('servers', default="[]"))+[text])

    def on_file_click(self):
        print(self.listwidget.currentItem().text())

    def on_add_click(self):
        name = self.evo_name_edit.text()
        if is_invalid_evo_name(name):
            print('invalid evolution name.\r\n For savety reasons some names and characters are forbidden to avoid the accidental removal of files or folders')
        else:
            item = self.listwidget2.currentItem()
            if item is None:
                print('please select a server first')
            else:
                server_str = item.text()
                if os.path.exists(get_data_folder()+'/Evolution_Project_Clones/'+name):
                    print('evolution folder already exists')
                else:

                    valid_genomes = False
                    try:
                        sg=eval(self.start_genomes_edit.text())
                        for key in ['gen', 'score', 'id']:
                            for genome in sg:
                                if key in genome:
                                    genome.pop(key)
                                    print(key, 'removed from genomes')
                        self.start_genomes_edit.setText(str(sg))
                        valid_genomes = True
                    except:
                        print('error parsing start genomes')

                    if not valid_genomes:
                        print('error parsing start genomes')
                    else:

                        self.copy_project(name, server_str)

                        folder = get_epc_folder()+'/'+name+'/'
                        ssm = SimpleStorageManager(folder)
                        ssm.save_param('server', server_str)

                        ssm.save_param('name', name)
                        ssm.save_param('slave_file', self.slave_file_edit.text())
                        ssm.save_param('inactive_genome_info', self.inactive_genome_info_edit.text())
                        ssm.save_param('thread_number', self.thread_number_edit.text())
                        ssm.save_param('individual_count', self.individual_count_edit.text())
                        ssm.save_param('mutation', self.mutation_edit.text())
                        ssm.save_param('death_rate', self.death_rate_edit.text())
                        ssm.save_param('start_genomes', self.start_genomes_edit.text())
                        ssm.save_param('constraints', self.constraints_edit.text())
                        ssm.save_param('python_cmd', self.python_cmd_edit.text())
                        ssm.save_param('evo_options', self.evo_options_edit.text())

                        exec_file = """
from PymoNNto.Exploration.Evolution.Evolution import *
if __name__ == '__main__':
    evo = Evolution(name='#name#',
                slave_file='#slave_file#',
                individual_count=#individual_count#,
                mutation=#mutation#,
                death_rate=#death_rate#,
                constraints=#constraints#,
                inactive_genome_info=#inactive_genome_info#,
                start_genomes=#start_genomes#,
                devices={'multi_thread': #thread_number#},
                additional_evo_params=#evo_options#
                )
    
    if not evo.start(ui=False):
        evo.continue_evolution(ui=False)
"""

                        print('generate execute_evolution.py...')
                        exec_file = exec_file.replace('#name#', name)
                        exec_file = exec_file.replace('#slave_file#', self.slave_file_edit.text())
                        exec_file = exec_file.replace('#inactive_genome_info#', self.inactive_genome_info_edit.text())
                        exec_file = exec_file.replace('#thread_number#', self.thread_number_edit.text())
                        exec_file = exec_file.replace('#individual_count#', self.individual_count_edit.text())
                        exec_file = exec_file.replace('#mutation#', self.mutation_edit.text())
                        exec_file = exec_file.replace('#death_rate#', self.death_rate_edit.text())
                        exec_file = exec_file.replace('#start_genomes#', self.start_genomes_edit.text())
                        exec_file = exec_file.replace('#constraints#', self.constraints_edit.text())
                        exec_file = exec_file.replace('#evo_options#', self.evo_options_edit.text())

                        evo_file = folder + 'execute_evolution.py'

                        md_file = open(folder + 'execute_evolution.py', "w")
                        md_file.write(exec_file)
                        md_file.close()

                        print('saving evolution config...')

                        gene_keys = list(eval(self.start_genomes_edit.text())[0].keys())
                        ssm.save_param('gene_keys', gene_keys)

                        if 'ssh' in server_str:
                            user, host, password = split_ssh_user_host_password_string(server_str)
                            print('transfer execute_evolution.py to ' + host + '...')
                            ssh = get_ssh_connection(host, user, password)
                            scp = SCPClient(ssh.get_transport())
                            scp.put(evo_file, name + '/execute_evolution.py')
                            scp.close()
                            ssh.close()

                        print('adding tab...')
                        self.add_tab(name)

    def copy_project(self, evo_name, device_str):
        if device_str == 'local':
            clone_project(evo_name)

        if 'ssh' in device_str:
            user, host, password = split_ssh_user_host_password_string(device_str)
            if user is not None and host is not None:
                transfer_project(evo_name, user, host, password)

            Data_Folder = get_data_folder()
            if Data_Folder != './Data':
                epc_folder = Data_Folder + '/Evolution_Project_Clones'
                create_folder_if_not_exist(epc_folder)
                run_folder = epc_folder + '/' + evo_name
                create_folder_if_not_exist(run_folder)


if __name__ == '__main__':
    UI_Evolution_Manager().show()

